##############################################################################
#
# Copyright (c) 2004-2008 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Test harness for the test runner itself.
"""

import doctest
import gc
import io
import os
import re
import sys
import unittest

from zope.testing import renormalizing

from ..util import uses_refcounts


# separated checkers for the different platform,
# because it s...s to maintain just one
if sys.platform == 'win32':
    checker = renormalizing.RENormalizing([
        # rewrite pdb prompt for coverage runs:
        (re.compile('->None'), ''),

        (re.compile(r"<module>"), (r'?')),

        # testtools content formatter is used to mime-encode
        # tracebacks when the SubunitOutputFormatter is used, and the
        # resulting text includes a size which can vary depending on
        # the path included in the traceback.
        (re.compile(r'traceback\n[A-F\d]+', re.MULTILINE),
         r'traceback\nNNN'),

        # hopefully, we'll make Windows happy
        # replaces drives with nothing
        (re.compile("'[A-Za-z]:\\\\"), "'"),

        # more Windows happiness
        # double backslashes in coverage???
        (re.compile(r'\\\\'), '/'),

        # even more Windows happiness
        # replaces backslashes in paths
        (re.compile(r'\\'), '/'),

        (re.compile(r'/r$', re.MULTILINE), '\\r'),  # undo some of that

        # this is a magic to put linefeeds into the doctest
        (re.compile('##r##\n'), '\r'),

        (re.compile(r'(\d+ minutes )?\d+[.]\d\d\d seconds'), 'N.NNN seconds'),
        (re.compile(r'\d+[.]\d\d\d s'), 'N.NNN s'),
        (re.compile(r'\d+[.]\d\d\d{'), 'N.NNN{'),
        (re.compile(r'\d{4}-\d\d-\d\d \d\d:\d\d:\d\d\.\d+'),
         'YYYY-MM-DD HH:MM:SS.mmmmmm'),
        (re.compile('( |")[^\n]+testrunner-ex'), r'\1testrunner-ex'),
        (re.compile('( |")[^\n]+testrunner.py'), r'\1testrunner.py'),
        (re.compile(r'> [^\n]*(doc|unit)test[.]py\(\d+\)'),
         r'\1test.py(NNN)'),
        (re.compile(r'[.]py\(\d+\)'), r'.py(NNN)'),
        (re.compile(r'[.]py:\d+'), r'.py:NNN'),
        (re.compile(r' line \d+,', re.IGNORECASE), r' Line NNN,'),
        (re.compile(r' line {([a-z]+)}\d+{', re.IGNORECASE),
         r' Line {\1}NNN{'),

        # omit traceback entries for unittest.py or doctest.py (and
        # their package variants) from output:
        (re.compile(r'^ +File "[^\n]*(doctest|unittest|case)(/__init__)?.py", '
                    r'[^\n]+\n[^\n]+\n',
                    re.MULTILINE),
         r''),
        (re.compile(r'^{\w+} +File "{\w+}[^\n]*(doctest|unittest|case)'
                    r'(/__init__)?.py{\w+}", [^\n]+\n[^\n]+\n',
                    re.MULTILINE),
         r''),
        # (re.compile('^> [^\n]+->None$', re.M), '> ...->None'),

        # disregard trailing whitespace
        (re.compile(r'\s+$'), ''),

        # Python 3.13+ no longer prints this `--Return--` line in pdb tests.
        (re.compile(r'--Return--'), ''),
    ])
else:
    # *nix
    checker = renormalizing.RENormalizing([
        # rewrite pdb prompt for coverage runs:
        (re.compile('->None'), ''),

        (re.compile(r"<module>"), (r'?')),

        # this is a magic to put linefeeds into the doctest
        # on win it takes one step, linux is crazy about the same...
        (re.compile('##r##'), r'\r'),
        (re.compile(r'\r'), '\\\\r\n'),

        (re.compile(r'(\d+ minutes )?\d+[.]\d\d\d seconds'), 'N.NNN seconds'),
        (re.compile(r'\d+[.]\d\d\d s'), 'N.NNN s'),
        (re.compile(r'\d+[.]\d\d\d{'), 'N.NNN{'),
        (re.compile(r'\d{4}-\d\d-\d\d \d\d:\d\d:\d\d\.\d+'),
         'YYYY-MM-DD HH:MM:SS.mmmmmm'),
        (re.compile('( |"|\')[^\'\n]+testrunner-ex'), r'\1testrunner-ex'),
        (re.compile('( |"|\')[^\'\n]+testrunner.py'), r'\1testrunner.py'),
        (re.compile(r'> [^\n]*(doc|unit)test[.]py\(\d+\)'),
         r'\1test.py(NNN)'),
        (re.compile(r'[.]py\(\d+\)'), r'.py(NNN)'),
        (re.compile(r'[.]py:\d+'), r'.py:NNN'),
        (re.compile(r' line \d+,', re.IGNORECASE), r' Line NNN,'),
        (re.compile(r' line {([a-z]+)}\d+{', re.IGNORECASE),
         r' Line {\1}NNN{'),

        # testtools content formatter is used to mime-encode
        # tracebacks when the SubunitOutputFormatter is used, and the
        # resulting text includes a size which can vary depending on
        # the path included in the traceback.
        (re.compile(r'traceback\n[A-F\d]+', re.MULTILINE),
         r'traceback\nNNN'),

        # omit traceback entries for unittest.py or doctest.py (and
        # their package variants) from output:
        (re.compile(r'^ +File "[^\n]*(doctest|unittest|case)(/__init__)?.py", '
                    r'[^\n]+\n[^\n]+\n',
                    re.MULTILINE),
         r''),
        (re.compile(r'^{\w+} +File "{\w+}[^\n]*(doctest|unittest|case)'
                    r'(/__init__)?.py{\w+}", [^\n]+\n[^\n]+\n',
                    re.MULTILINE),
         r''),
        # Python 3.13+ no longer prints this `--Return--` line in pdb tests.
        (re.compile(r'--Return--'), ''),

        # disregard trailing whitespace
        (re.compile(r'\s+$', re.MULTILINE), ''),
    ])


# Monkey-patch doctest with our own _SpoofOut replacement.  We
# need sys.stdout to be binary-capable so that we can pass through binary
# output from test formatters cleanly, which is in particular required for
# subunit.  We don't expect to be able to do actual binary comparisons in
# doctests, but that's OK.
# See https://github.com/zopefoundation/zope.testrunner/pull/23 for the
# background.

class _SpoofOut(io.TextIOWrapper):
    def __init__(self):
        super().__init__(io.BytesIO(), encoding='utf-8')

    def write(self, s):
        super().write(s)
        # Always flush immediately so that getvalue() never returns
        # short results.
        self.flush()

    def getvalue(self):
        result = self.buffer.getvalue().decode('utf-8', 'replace')
        # If anything at all was written, make sure there's a trailing
        # newline.  There's no way for the expected output to indicate
        # that a trailing newline is missing.
        if result and not result.endswith("\n"):
            result += "\n"
        # We're reading bytes, so we have to do universal newlines
        # conversion by hand.
        return result.replace(os.linesep, '\n')

    def truncate(self, size=None):
        self.seek(size)
        super().truncate()


def setUp(test):
    test.globs['saved-sys-info'] = (
        sys.path[:],
        sys.argv[:],
        sys.modules.copy(),
    )
    if hasattr(gc, 'get_threshold'):
        test.globs['saved-gc-threshold'] = gc.get_threshold()
    test.globs['saved-doctest-SpoofOut'] = doctest._SpoofOut
    doctest._SpoofOut = _SpoofOut
    test.globs['this_directory'] = os.path.split(__file__)[0]
    test.globs['testrunner_script'] = sys.argv[0]


def tearDown(test):
    sys.path[:], sys.argv[:] = test.globs['saved-sys-info'][:2]
    if hasattr(gc, 'get_threshold'):
        gc.set_threshold(*test.globs['saved-gc-threshold'])
    sys.modules.clear()
    sys.modules.update(test.globs['saved-sys-info'][2])
    doctest._SpoofOut = test.globs['saved-doctest-SpoofOut']


def test_suite():
    optionflags = (doctest.ELLIPSIS |
                   doctest.NORMALIZE_WHITESPACE |
                   doctest.REPORT_NDIFF)
    suites = [
        doctest.DocFileSuite(
            'testrunner-arguments.rst',
            'testrunner-coverage.rst',
            'testrunner-debugging-layer-setup.rst',
            'testrunner-debugging-import-failure.rst',
            'testrunner-debugging-nonprintable-exc.rst',
            'testrunner-debugging.rst',
            'testrunner-edge-cases.rst',
            'testrunner-errors.rst',
            'testrunner-layers-api.rst',
            'testrunner-layers-instances.rst',
            'testrunner-layers-buff.rst',
            'testrunner-subprocess-errors.rst',
            'testrunner-layers-cantfind.rst',
            'testrunner-layers-cwd.rst',
            'testrunner-layers-ntd.rst',
            'testrunner-layers-topological-sort.rst',
            'testrunner-layers.rst',
            'testrunner-progress.rst',
            'testrunner-colors.rst',
            'testrunner-simple.rst',
            'testrunner-nestedcode.rst',
            'testrunner-test-selection.rst',
            'testrunner-verbose.rst',
            'testrunner-repeat.rst',
            'testrunner-knit.rst',
            'testrunner-shuffle.rst',
            'testrunner-stops-when-stop-on-error.rst',
            'testrunner-new-threads.rst',
            'testrunner-subtest.rst',
            setUp=setUp, tearDown=tearDown,
            optionflags=optionflags,
            checker=checker),
        doctest.DocTestSuite('zope.testrunner'),
        doctest.DocTestSuite('zope.testrunner.coverage',
                             optionflags=optionflags),
        doctest.DocTestSuite('zope.testrunner.options'),
        doctest.DocTestSuite('zope.testrunner.find'),
    ]

    # PyPy uses a different garbage collector
    if hasattr(gc, 'get_threshold'):
        suites.append(
            doctest.DocFileSuite(
                'testrunner-gc.rst',
                setUp=setUp, tearDown=tearDown,
                optionflags=optionflags,
                checker=checker))

    # PyPy does not support sourceless imports, apparently (tried version 1.9)
    if 'PyPy' not in sys.version and not sys.dont_write_bytecode:
        suites.append(
            doctest.DocFileSuite(
                'testrunner-wo-source.rst',
                setUp=setUp, tearDown=tearDown,
                optionflags=optionflags,
                checker=checker))

    if sys.platform == 'win32':
        suites.append(
            doctest.DocFileSuite(
                'testrunner-coverage-win32.rst',
                setUp=setUp, tearDown=tearDown,
                optionflags=optionflags,
                checker=checker))

    suites.append(
        doctest.DocFileSuite(
            'testrunner-profiling.rst',
            setUp=setUp, tearDown=tearDown,
            optionflags=optionflags,
            checker=renormalizing.RENormalizing([
                (re.compile(r'tests_profile[.]\S*[.]prof'),
                 'tests_profile.*.prof'),
            ]),
        )
    )

    suites.append(
        doctest.DocFileSuite(
            'testrunner-profiling-cprofiler.rst',
            setUp=setUp, tearDown=tearDown,
            optionflags=optionflags,
            checker=renormalizing.RENormalizing([
                (re.compile(r'tests_profile[.]\S*[.]prof'),
                 'tests_profile.*.prof'),
            ]),
        )
    )

    suites.append(
        doctest.DocFileSuite(
            'testrunner-report-skipped.rst',
            setUp=setUp, tearDown=tearDown,
            optionflags=optionflags,
            checker=checker)
    )

    if hasattr(sys, 'gettotalrefcount'):
        suites.append(
            doctest.DocFileSuite(
                'testrunner-leaks.rst',
                setUp=setUp, tearDown=tearDown,
                optionflags=optionflags,
                checker=renormalizing.RENormalizing([
                    (re.compile(r'(\d+ minutes )?\d+[.]\d\d\d seconds'),
                     'N.NNN seconds'),
                    (re.compile(r'sys refcount=\d+ +change=\d+'),
                     'sys refcount=NNNNNN change=NN'),
                    (re.compile(r'sum detail refcount=\d+ +'),
                     'sum detail refcount=NNNNNN '),
                    (re.compile(r'total +\d+ +\d+'),
                     'total               NNNN    NNNN'),
                    (re.compile(r"^ +(int|type) +-?\d+ +-?\d+ *\n", re.M),
                     ''),
                ]),
            )
        )
    else:
        suites.append(
            doctest.DocFileSuite(
                'testrunner-leaks-err.rst',
                setUp=setUp, tearDown=tearDown,
                optionflags=optionflags,
                checker=checker,
            )
        )
    if uses_refcounts:
        suites.append(
            doctest.DocFileSuite(
                'testrunner-gc-after-test.rst',
                setUp=setUp, tearDown=tearDown,
                optionflags=optionflags,
                checker=renormalizing.RENormalizing([
                    (re.compile(r'(\d+ minutes )?\d+[.]\d\d\d seconds'),
                     'N.NNN seconds'),
                    (re.compile(r'\(\d+[.]\d\d\d s\)'),
                     '(N.NNN s)'),
                    # objects on cycle differ between different python versions
                    (re.compile(r'\[\d+\]'), '[C]')])))

    try:
        import subunit
        subunit
    except ImportError:
        suites.append(
            doctest.DocFileSuite(
                'testrunner-subunit-err.rst',
                setUp=setUp, tearDown=tearDown,
                optionflags=optionflags,
                checker=checker))
    else:
        suites.append(
            doctest.DocFileSuite(
                'testrunner-subunit.rst',
                'testrunner-subunit-v2.rst',
                setUp=setUp, tearDown=tearDown,
                optionflags=optionflags,
                checker=checker))
        if hasattr(sys, 'gettotalrefcount'):
            suites.append(
                doctest.DocFileSuite(
                    'testrunner-subunit-leaks.rst',
                    setUp=setUp, tearDown=tearDown,
                    optionflags=optionflags,
                    checker=checker))

    suites.append(doctest.DocFileSuite(
        'testrunner-unexpected-success.rst',
        setUp=setUp, tearDown=tearDown,
        optionflags=optionflags,
        checker=checker))

    return unittest.TestSuite(suites)



##############################################################################
#
# Copyright (c) 2020 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""A large number of sample tests."""

import unittest


class Layer1:
    """A layer that can't be torn down."""

    @classmethod
    def setUp(self):
        pass

    @classmethod
    def tearDown(self):
        raise NotImplementedError


class Layer2:

    @classmethod
    def setUp(self):
        pass

    @classmethod
    def tearDown(self):
        pass


class TestNoTeardown(unittest.TestCase):

    layer = Layer1

    def test_something(self):
        pass


def make_TestMany():
    attrs = {'layer': Layer2}
    # Add enough failing test methods to make the concatenation of all their
    # test IDs (formatted as "test_foo (sampletests_many.TestMany)")
    # overflow the capacity of a pipe.  This is system-dependent, but on
    # Linux since 2.6.11 it defaults to 65536 bytes, so will overflow by the
    # time we've written 874 of these test IDs.  If the pipe capacity is
    # much larger than that, then this test might be ineffective.
    for i in range(1000):
        attrs['test_some_very_long_test_name_with_padding_%03d' % i] = (
            lambda self: self.fail())
    return type('TestMany', (unittest.TestCase,), attrs)


TestMany = make_TestMany()


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(
        unittest.defaultTestLoader.loadTestsFromTestCase(TestNoTeardown))
    suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(TestMany))
    return suite


if __name__ == '__main__':
    unittest.main()



import threading
import time
import unittest


class Mythread(threading.Thread):
    # The default includes a timestamp when the thread was started.
    def __repr__(self):
        return "<Thread(%s)>" % self.name


class TestNewThreadsReporting(unittest.TestCase):
    def test_leave_thread_behind(self):
        Mythread(name='t1', target=time.sleep, args=[1]).start()



##############################################################################
#
# Copyright (c) 2004-2008 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Subprocess support.
"""

import sys

import zope.testrunner.feature


class SubProcess(zope.testrunner.feature.Feature):
    """Lists all tests in the report instead of running the tests."""

    def __init__(self, runner):
        super().__init__(runner)
        self.active = bool(runner.options.resume_layer)

    def global_setup(self):
        self.original_stderr = sys.stderr
        sys.stderr = sys.stdout
        if self.runner.options.processes > 1:
            # If we only have one subprocess, there's absolutely
            # no reason to squelch.  We will let the messages through in a
            # timely manner, if they have been requested. On the other hand, if
            # there are multiple processes, we do squelch to 0.
            self.runner.options.verbose = 0
        self.progress = False

    def report(self):
        sys.stdout.close()
        # Communicate with the parent.  The protocol is obvious:
        print(self.runner.ran,
              len(self.runner.failures), len(self.runner.errors),
              file=self.original_stderr)
        for test, exc_info in self.runner.failures:
            print(' '.join(str(test).strip().split('\n')),
                  file=self.original_stderr)
        for test, exc_info in self.runner.errors:
            print(' '.join(str(test).strip().split('\n')),
                  file=self.original_stderr)
        self.original_stderr.flush()



##############################################################################
#
# Copyright (c) 2022 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Directed graph
"""

from itertools import count


class DiGraph:
    """Directed graph.

    A directed graph is a set of nodes together with a
    neighboring relation.

    The class makes intensive use of dicts; therefore, hashability
    is important. Therefore, the class usually does not work
    with the nodes directly but transforms them via
    a ``make_hashable`` function, ``id`` by default.
    This works well for object types where equality is identity.
    For other types, you may need to deactive the transformation
    or use a different ``make_hashable``.
    """

    def __init__(self, nodes=None, make_hashable=id):
        self._nodes = set()  # transformed nodes
        self._neighbors = {}  # node --> neighbors  -- transformed
        if make_hashable:
            tr2n = {}  # transform -> node

            tr_node = make_hashable

            def utr_node(node):
                return tr2n[node]

            def tr_nodes(nodes):
                ns = set()
                add = ns.add
                for n in nodes:
                    trn = make_hashable(n)
                    if trn not in tr2n:
                        tr2n[trn] = n
                    add(trn)
                return ns

        else:
            def tr_nodes(nodes): return set(nodes)  # noqa: E731
            utr_node = tr_node = lambda node: node

        self._transform_node = tr_node
        self._transform_nodes = tr_nodes
        self._untransform_node = utr_node

        if nodes is not None:
            self.add_nodes(nodes)

    def add_nodes(self, nodes):
        """add *nodes* (iterator) to the graph's nodes."""
        self._nodes |= self._transform_nodes(nodes)

    def add_neighbors(self, node, neighbors, ignore_unknown=True):
        """add *neighbors* (iterator) as neighbors for *node*.

        if *ignore_unknown*, unknown nodes in *neighbors* are
        ignored, otherwise a ``KeyError`` is raised.
        """
        tr_n = self._transform_node(node)
        nodes = self._nodes
        nbad = tr_n not in nodes
        if nbad:
            if ignore_unknown:
                return
            else:
                raise KeyError(node)
        tr_neighbors = self._transform_nodes(neighbors)
        known_neighbors = tr_neighbors & nodes
        if not ignore_unknown and len(known_neighbors) != len(tr_neighbors):
            raise KeyError(tr_neighbors - known_neighbors)
        nbs = self._neighbors.get(tr_n)
        if nbs is None:
            self._neighbors[tr_n] = known_neighbors
        else:
            nbs |= known_neighbors

    def nodes(self):
        """iterate of the graph's nodes."""
        utr = self._untransform_node
        for n in self._nodes:
            yield utr(n)

    def neighbors(self, node):
        """iterate over *node*'s neighbors."""
        utr = self._untransform_node
        for n in self._neighbors.get(self._transform_node(node), ()):
            yield utr(n)

    def sccs(self, trivial=False):
        """iteratate over the strongly connected components.

        If *trivial*, include the trivial components; otherwise
        only the cycles.

        This is an implementation of the "Tarjan SCC" algorithm.
        """

        # any node is either in ``unvisited`` or in ``state``
        unvisited = self._nodes.copy()
        state = {}  # nodes -> state

        ancestors = []  # the ancestors of the currently processed node
        stack = []  # the nodes which might still be on a cycle

        # the algorithm visits each node twice in a depth first order
        # In the first visit, visits for the unprocessed neighbors
        # are scheduled as well as the second visit to this
        # node after all neighbors have been processed.
        dfs = count()  # depth first search visit order
        rtn_marker = object()  # marks second visit to ``ancestor`` top
        visits = []  # scheduled visits

        while unvisited:
            node = next(iter(unvisited))
            # determine the depth first spanning tree rooted in *node*
            visits.append(node)
            while visits:
                visit = visits[-1]  # ``rtn_marker`` or node
                if visit is rtn_marker:
                    # returned to the top of ``ancestors``
                    visits.pop()
                    node = ancestors.pop()  # returned to *node*
                    nstate = state[node]
                    if nstate.low == nstate.dfs:
                        # SCC root
                        scc = []
                        while True:
                            n = stack.pop()
                            state[n].stacked = False
                            scc.append(n)
                            if n is node:
                                break
                        if len(scc) == 1 and not trivial:
                            # check for triviality
                            n = scc[0]
                            if n not in self._neighbors[n]:
                                continue  # tivial -- ignore
                        utr = self._untransform_node
                        yield [utr(n) for n in scc]
                    if not ancestors:
                        # dfs tree determined
                        assert not visits
                        break
                    pstate = state[ancestors[-1]]
                    nstate = state[node]
                    low = nstate.low
                    if low < pstate.low:
                        pstate.low = low
                else:  # scheduled first visit
                    node = visit
                    nstate = state.get(node)
                    if nstate is not None:
                        # we have already been visited
                        if nstate.stacked:
                            # update parent
                            pstate = state[ancestors[-1]]
                            if nstate.dfs < pstate.low:
                                pstate.low = nstate.dfs
                        visits.pop()
                        continue
                    unvisited.remove(node)
                    nstate = state[node] = _TarjanState(dfs)
                    ancestors.append(node)
                    stack.append(node)
                    nstate.stacked = True
                    visits[-1] = rtn_marker  # schedule return visit
                    # schedule neighbor visits
                    visits.extend(self._neighbors.get(node, ()))


class _TarjanState:
    """representation of a node's processing state."""
    __slots__ = "stacked dfs low".split()

    def __init__(self, dfs):
        self.stacked = False
        self.dfs = self.low = next(dfs)

    def __repr__(self):
        return "dfs=%d low=%d stacked=%s" \
               % (self.dfs, self.low, self.stacked)



##############################################################################
#
# Copyright (c) 2004-2008 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Test execution
"""
import errno
import gc
import io
import os
import pprint
import queue
import re
import subprocess
import sys
import threading
import time
import traceback
import unittest
import warnings
from contextlib import contextmanager
from io import StringIO
from pathlib import Path

import zope.testrunner
import zope.testrunner._doctest
import zope.testrunner.coverage
import zope.testrunner.debug
import zope.testrunner.filter
import zope.testrunner.garbagecollection
import zope.testrunner.interfaces
import zope.testrunner.listing
import zope.testrunner.logsupport
import zope.testrunner.process
import zope.testrunner.profiling
import zope.testrunner.selftest
import zope.testrunner.shuffle
import zope.testrunner.statistics
import zope.testrunner.tb_format
from zope.testrunner import threadsupport
from zope.testrunner.find import _layer_name_cache
from zope.testrunner.find import import_name
from zope.testrunner.find import name_from_layer
from zope.testrunner.formatter import XMLOutputFormattingWrapper
from zope.testrunner.layer import EmptyLayer
from zope.testrunner.layer import EmptySuite
from zope.testrunner.layer import UnitTests
from zope.testrunner.options import get_options
from zope.testrunner.refcount import TrackRefs

from .digraph import DiGraph
from .util import is_jython
from .util import uses_refcounts


class UnexpectedSuccess(Exception):
    pass


PYREFCOUNT_PATTERN = re.compile(r'\[[0-9]+ refs\]')


class SubprocessError(Exception):
    """An error occurred when running a subprocess
    """

    def __init__(self, reason, stderr):
        self.reason = reason
        self.stderr = stderr

    def __str__(self):
        return f'{self.reason}: {self.stderr}'


class CanNotTearDown(Exception):
    "Couldn't tear down a test"


class Runner:
    """The test runner.

    It is the central point of this package and responsible for finding and
    executing tests as well as configuring itself from the (command-line)
    options passed into it.

    .. versionchanged:: 4.8.0
       Add the *warnings* keyword argument. If this is ``None`` (the default)
       and the user hasn't configured Python otherwise with command-line
       arguments or environment variables, we will enable the default
       warnings, including ``DeprecationWarning``, when running tests.
       Otherwise, it can be any string acceptable to
       :func:`warnings.simplefilter` and that filter will be in effect while
       running tests.

    """

    def __init__(self, defaults=None, args=None, found_suites=None,
                 options=None, script_parts=None, cwd=None,
                 warnings=None):
        if defaults is None:
            self.defaults = []
        else:
            self.defaults = defaults
        self.args = args
        self.found_suites = found_suites
        self.options = options
        self.script_parts = script_parts
        self.cwd = cwd
        self.failed = True
        if warnings is None and not sys.warnoptions:
            # even if DeprecationWarnings are ignored by default
            # print them anyway unless other warnings settings are
            # specified by the warnings arg or the -W python flag
            self.warnings = 'default'
        else:
            # here self.warnings is set either to the value passed
            # to the warnings args or to None.
            # If the user didn't pass a value self.warnings will
            # be None. This means that the behavior is unchanged
            # and depends on the values passed to -W.
            self.warnings = warnings

        self.ran = 0
        self.skipped = []
        self.failures = []
        self.errors = []
        self.import_errors = []

        self.show_report = True
        self.do_run_tests = True

        self.features = []

        self.tests_by_layer_name = {}

    def ordered_layers(self):
        if (self.options.processes > 1 and not self.options.resume_layer):
            # if we want multiple processes, we need a fake layer as first
            # to start spreading out layers/tests to subprocesses
            # but only if this is not in the subprocess
            yield (name_from_layer(EmptyLayer), EmptyLayer, EmptySuite())

        layer_names = {layer_from_name(layer_name): layer_name
                       for layer_name in self.tests_by_layer_name}
        for layer in order_by_bases(layer_names):
            layer_name = layer_names[layer]
            yield layer_name, layer, self.tests_by_layer_name[layer_name]

    def register_tests(self, tests):
        """Registers tests."""
        # XXX To support multiple features that find tests this shouldn't be
        # an update but merge the various layers individually.
        self.tests_by_layer_name.update(tests)

    def run(self):
        self.configure()
        if self.options.fail:
            return True

        # XXX Hacky to support existing code.
        self.layer_name_cache = _layer_name_cache
        self.layer_name_cache.clear()

        with self._enabled_warnings():
            # Enable warnings during setup so that
            # warnings raised on import (which we do for test
            # discover) can be reported.

            # Global setup
            for feature in self.features:
                feature.global_setup()

            # Late setup
            #
            # Some system tools like profilers are really bad with stack
            # frames.  E.g. hotshot doesn't like it when we leave the stack
            # frame that we called start() from.
            for feature in self.features:
                feature.late_setup()

            try:
                if self.do_run_tests:
                    self.run_tests()
            finally:
                # Early teardown
                for feature in reversed(self.features):
                    feature.early_teardown()
                # Global teardown
                for feature in reversed(self.features):
                    feature.global_teardown()

            if self.show_report:
                for feature in self.features:
                    feature.report()

        if self.options.xmlOutput:
            self.options.output.writeXMLReports()

    def configure(self):
        if self.args is None:
            self.args = sys.argv[:]
        # Check to see if we are being run as a subprocess. If we are,
        # then use the resume-layer and defaults passed in.
        if len(self.args) > 1 and self.args[1] == '--resume-layer':
            self.args.pop(1)
            resume_layer = self.args.pop(1)
            resume_number = int(self.args.pop(1))
            self.defaults = []
            while len(self.args) > 1 and self.args[1] == '--default':
                self.args.pop(1)
                self.defaults.append(self.args.pop(1))

            sys.stdin = FakeInputContinueGenerator()
        else:
            resume_layer = resume_number = None

        options = get_options(self.args, self.defaults)

        options.testrunner_defaults = self.defaults
        options.resume_layer = resume_layer
        options.resume_number = resume_number

        if options.xmlOutput:
            folder = Path(options.xmlOutput).resolve()
            folder.mkdir(parents=True, exist_ok=True)
            options.output = XMLOutputFormattingWrapper(
                options.output, folder=folder)

        self.options = options

        self.features.append(zope.testrunner.selftest.SelfTest(self))
        self.features.append(zope.testrunner.logsupport.Logging(self))
        self.features.append(zope.testrunner.coverage.Coverage(self))
        self.features.append(zope.testrunner._doctest.DocTest(self))
        self.features.append(zope.testrunner.profiling.Profiling(self))
        if is_jython:
            # Jython GC support is not yet implemented
            pass
        else:
            self.features.append(
                zope.testrunner.garbagecollection.Threshold(self))
            self.features.append(
                zope.testrunner.garbagecollection.Debug(self))

        self.features.append(zope.testrunner.find.Find(self))
        self.features.append(zope.testrunner.shuffle.Shuffle(self))
        self.features.append(zope.testrunner.process.SubProcess(self))
        self.features.append(zope.testrunner.filter.Filter(self))
        self.features.append(zope.testrunner.listing.Listing(self))
        self.features.append(
            zope.testrunner.statistics.Statistics(self))
        self.features.append(zope.testrunner.tb_format.Traceback(self))

        # Remove all features that aren't activated
        self.features = [f for f in self.features if f.active]

    @contextmanager
    def _enabled_warnings(self):
        """
        A context manager to enable warnings as configured.
        """
        with warnings.catch_warnings():
            if self.warnings:
                # if self.warnings is set, use it to filter all the warnings
                warnings.simplefilter(self.warnings)
                # if the filter is 'default' or 'always', special-case the
                # warnings from the deprecated unittest methods to show them
                # no more than once per module, because they can be fairly
                # noisy.  The -Wd and -Wa flags can be used to bypass this
                # only when self.warnings is None.
                if self.warnings in ['default', 'always']:
                    warnings.filterwarnings(
                        'module',
                        category=DeprecationWarning,
                        message=r'Please use assert\w+ instead.')
            yield

    def run_tests(self):
        """Run all tests that were registered.

        Returns True if there where failures or False if all tests passed.

        """
        setup_layers = {}
        layers_to_run = list(self.ordered_layers())
        should_resume = False

        while layers_to_run:
            layer_name, layer, tests = layers_to_run[0]
            for feature in self.features:
                feature.layer_setup(layer)
            try:
                self.ran += run_layer(self.options, layer_name, layer, tests,
                                      setup_layers, self.failures, self.errors,
                                      self.skipped, self.import_errors)
            except zope.testrunner.interfaces.EndRun:
                self.failed = True
                break
            except CanNotTearDown:
                if not self.options.resume_layer:
                    should_resume = True
                    break

            layers_to_run.pop(0)
            if self.options.processes > 1:
                should_resume = True
                break

            if self.options.stop_on_error and (self.failures or self.errors):
                break

        if should_resume:
            if layers_to_run:
                self.ran += resume_tests(
                    self.script_parts, self.options, self.features,
                    layers_to_run, self.failures, self.errors,
                    self.skipped, self.cwd)

        if setup_layers:
            if self.options.resume_layer is None:
                self.options.output.info("Tearing down left over layers:")
            tear_down_unneeded(
                self.options, (), setup_layers, self.errors, optional=True)

        self.failed = bool(self.import_errors or self.failures or self.errors)


def handle_layer_failure(failure_type, output, errors):
    if hasattr(output, 'layer_failure'):
        output.layer_failure(failure_type.subunit_label, sys.exc_info())
    else:
        f = StringIO()
        traceback.print_exc(file=f)
        output.error(f.getvalue())
    errors.append((failure_type, sys.exc_info()))


def run_tests(options, tests, name, failures, errors, skipped, import_errors):
    repeat = options.repeat or 1
    repeat_range = iter(range(repeat))
    ran = 0

    output = options.output

    if is_jython:
        # Jython has no GC suppport - set count to 0
        lgarbage = 0
    else:
        gc.collect()
        lgarbage = len(gc.garbage)

    if options.report_refcounts:
        if options.verbose:
            # XXX This code path is untested
            track = TrackRefs()
        rc = sys.gettotalrefcount()

    for iteration in repeat_range:
        if repeat > 1:
            output.info("Iteration %d" % (iteration + 1))

        if options.verbose > 0 or options.progress:
            output.info('  Running:')
        result = TestResult(options, tests, layer_name=name)

        t = time.time()

        if options.post_mortem:
            # post-mortem debugging
            for test in tests:
                if result.shouldStop:
                    break
                result.startTest(test)
                state = test.__dict__.copy()
                try:
                    try:
                        test.debug()
                    except KeyboardInterrupt:
                        raise
                    except unittest.SkipTest as e:
                        result.addSkip(test, str(e))
                    except BaseException:
                        result.addError(
                            test,
                            sys.exc_info()[:2] + (sys.exc_info()[2].tb_next, ),
                        )
                    else:
                        result.addSuccess(test)
                finally:
                    result.stopTest(test)
                test.__dict__.clear()
                test.__dict__.update(state)

        else:
            # normal
            for test in tests:
                if result.shouldStop:
                    break
                state = test.__dict__.copy()
                test(result)
                test.__dict__.clear()
                test.__dict__.update(state)

        t = time.time() - t
        output.stop_tests()
        failures.extend(result.failures)
        n_failures = len(result.failures)
        failures.extend(result.unexpectedSuccesses)
        n_failures += len(result.unexpectedSuccesses)
        skipped.extend(result.skipped)
        errors.extend(result.errors)
        output.summary(n_tests=result.testsRun,
                       n_failures=n_failures,
                       n_errors=len(result.errors) + len(import_errors),
                       n_seconds=t,
                       n_skipped=len(result.skipped))
        ran = result.testsRun

        if is_jython:
            lgarbage = 0
        else:
            gc.collect()
            if len(gc.garbage) > lgarbage:
                output.garbage(gc.garbage[lgarbage:])
                lgarbage = len(gc.garbage)

        if options.report_refcounts:

            # If we are being tested, we don't want stdout itself to
            # foul up the numbers. :)
            try:
                sys.stdout.getvalue()
            except AttributeError:
                pass

            prev = rc
            rc = sys.gettotalrefcount()
            if options.verbose:
                track.update()
                if iteration > 0:
                    output.detailed_refcounts(track, rc, prev)
                else:
                    track.delta = None
            elif iteration > 0:
                output.refcounts(rc, prev)

    return ran


def run_layer(options, layer_name, layer, tests, setup_layers,
              failures, errors, skipped, import_errors):

    output = options.output
    gathered = []
    gather_layers(layer, gathered)
    needed = {ly: 1 for ly in gathered}
    if options.resume_number != 0:
        output.info("Running %s tests:" % layer_name)
    tear_down_unneeded(options, needed, setup_layers, errors)

    if options.resume_layer is not None:
        output.info_suboptimal("  Running in a subprocess.")

    try:
        setup_layer(options, layer, setup_layers)
    except zope.testrunner.interfaces.EndRun:
        raise
    except MemoryError:
        raise
    except Exception:
        handle_layer_failure(SetUpLayerFailure(layer), output, errors)
        return 0
    else:
        return run_tests(options, tests, layer_name, failures, errors, skipped,
                         import_errors)


class SetUpLayerFailure(unittest.TestCase):

    subunit_label = 'setUp'

    def __init__(self, layer):
        super().__init__()
        self.layer = layer

    def runTest(self):
        pass

    def __str__(self):
        return "Layer: %s.setUp" % (name_from_layer(self.layer))


class TearDownLayerFailure(unittest.TestCase):

    subunit_label = 'tearDown'

    def __init__(self, layer):
        super().__init__()
        self.layer = layer

    def runTest(self):
        pass

    def __str__(self):
        return "Layer: %s.tearDown" % (name_from_layer(self.layer))


def spawn_layer_in_subprocess(result, script_parts, options, features,
                              layer_name, layer, failures, errors, skipped,
                              resume_number, cwd=None):
    output = options.output
    child = None
    try:
        # BBB
        if script_parts is None:
            script_parts = zope.testrunner._script_parts()
        args = [sys.executable]
        args.extend(script_parts)
        args.extend(['--resume-layer', layer_name, str(resume_number)])
        for d in options.testrunner_defaults:
            args.extend(['--default', d])

        args.extend(options.original_testrunner_args[1:])

        debugargs = args  # save them before messing up for windows
        if sys.platform.startswith('win'):
            args = args[0] + ' ' + ' '.join([
                ('"' + a.replace('\\', '\\\\').replace('"', '\\"') + '"')
                for a in args[1:]])

        for feature in features:
            feature.layer_setup(layer)

        child = subprocess.Popen(
            args, shell=False, stdin=subprocess.PIPE,
            stdout=subprocess.PIPE, stderr=subprocess.PIPE, cwd=cwd,
            close_fds=not sys.platform.startswith('win'))

        def reader_thread(f, buf):
            buf.append(f.read())

        # Start reading stderr in a thread.  This means we don't hang if the
        # subprocess writes more to stderr than the pipe capacity.
        stderr_buf = []
        stderr_thread = threading.Thread(
            target=reader_thread, args=(child.stderr, stderr_buf))
        stderr_thread.daemon = True
        stderr_thread.start()

        while True:
            try:
                while True:
                    # We use readline() instead of iterating over stdout
                    # because it appears that iterating over stdout causes a
                    # lot more buffering to take place (probably so it can
                    # return its lines as a batch). We don't want too much
                    # buffering because this foils automatic and human monitors
                    # trying to verify that the subprocess is still alive.
                    line = child.stdout.readline()
                    if not line:
                        break
                    result.write(line)
            except OSError as e:
                if e.errno == errno.EINTR:
                    # If the subprocess dies before we finish reading its
                    # output, a SIGCHLD signal can interrupt the reading.
                    # The correct thing to to in that case is to retry.
                    continue
                output.error(
                    "Error reading subprocess output for %s" % layer_name)
                output.info(str(e))
            else:
                break

        # Now we should be able to finish reading stderr.
        stderr_thread.join()
        errlines = stderr_buf[0].splitlines()
        erriter = iter(errlines)
        nfail = nerr = 0
        for line in erriter:
            try:
                result.num_ran, nfail, nerr = map(int, line.strip().split())
            except ValueError:
                continue
            else:
                break
        else:
            errmsg = "Could not communicate with subprocess!"
            errors.append(("subprocess for %s" % layer_name, None))
            if options.verbose >= 1:
                errmsg += "\nChild command line: %s" % debugargs
            if (options.verbose >= 2 or
                    (options.verbose == 1 and len(errlines) < 20)):
                errmsg += ("\nChild stderr was:\n" +
                           "\n".join("  " + line.decode('utf-8', 'replace')
                                     for line in errlines))
            elif options.verbose >= 1:
                errmsg += ("\nChild stderr was:\n" +
                           "\n".join("  " + line.decode('utf-8', 'replace')
                                     for line in errlines[:10]) +
                           "\n...\n" +
                           "\n".join("  " + line.decode('utf-8', 'replace')
                                     for line in errlines[-10:]))
            output.error_with_banner(errmsg)

        while nfail > 0:
            nfail -= 1
            # Doing erriter.next().strip() confuses the 2to3 fixer, so
            # we need to do it on a separate line. Also, in python 3 this
            # returns bytes, so we decode it.
            next_fail = next(erriter)
            failures.append((next_fail.strip().decode(), None))
        while nerr > 0:
            nerr -= 1
            # Doing erriter.next().strip() confuses the 2to3 fixer, so
            # we need to do it on a separate line. Also, in python 3 this
            # returns bytes, so we decode it.
            next_err = next(erriter)
            errors.append((next_err.strip().decode(), None))

    finally:
        result.done = True
        if child is not None:
            # Regardless of whether the process ran to completion, we
            # must properly cleanup the process to avoid
            # `ResourceWarning: subprocess XXX is still alive` and
            # `ResourceWarning: unclosed file` for its stdout and
            # stderr.
            child.kill()
            child.communicate()


def _get_output_buffer(stream):
    """Get a binary-safe version of a stream."""
    try:
        fileno = stream.fileno()
    except (io.UnsupportedOperation, AttributeError):
        pass
    else:
        # Win32 mangles \r\n to \n and that breaks streams.  See
        # https://bugs.launchpad.net/bugs/505078.
        if sys.platform == 'win32':
            import msvcrt
            msvcrt.setmode(fileno, os.O_BINARY)
    try:
        stream.write(b'')
    except TypeError:
        return stream.buffer
    return stream


class AbstractSubprocessResult:
    """A result of a subprocess layer run."""

    num_ran = 0
    done = False

    def __init__(self, layer_name, queue):
        self.layer_name = layer_name
        self.queue = queue
        self.stdout = []

    def write(self, out):
        """Receive a line of the subprocess out."""


class DeferredSubprocessResult(AbstractSubprocessResult):
    """Keeps stdout around for later processing,"""

    def write(self, out):
        if not _is_dots(out):
            self.stdout.append(out)


class ImmediateSubprocessResult(AbstractSubprocessResult):
    """Sends complete output to queue."""

    def __init__(self, layer_name, queue):
        super().__init__(layer_name, queue)
        self.stream = _get_output_buffer(sys.stdout)

    def write(self, out):
        self.stream.write(out)
        # Help keep-alive monitors (human or automated) keep up-to-date.
        self.stream.flush()


_is_dots = re.compile(br'\.+(\r\n?|\n)').match  # Windows sneaks in a \r\n.


class KeepaliveSubprocessResult(AbstractSubprocessResult):
    "Keeps stdout for later processing; sends marks to queue to show activity."

    _done = False

    def _set_done(self, value):
        self._done = value
        assert value, 'Internal error: unexpectedly setting done to False'
        self.queue.put((self.layer_name, ' LAYER FINISHED'))
    done = property(lambda self: self._done, _set_done)

    def write(self, out):
        if _is_dots(out):
            self.queue.put((self.layer_name, out.strip()))
        else:
            self.stdout.append(out)


def resume_tests(script_parts, options, features, layers, failures, errors,
                 skipped, cwd=None):
    results = []
    stdout_queue = None
    if options.processes == 1:
        result_factory = ImmediateSubprocessResult
    elif (options.verbose > 1 and
            not options.subunit and not options.subunit_v2):
        result_factory = KeepaliveSubprocessResult
        stdout_queue = queue.Queue()
    else:
        result_factory = DeferredSubprocessResult
    resume_number = int(options.processes > 1)
    ready_threads = []
    for layer_name, layer, tests in layers:
        result = result_factory(layer_name, stdout_queue)
        results.append(result)
        ready_threads.append(threading.Thread(
            target=spawn_layer_in_subprocess,
            args=(result, script_parts, options, features, layer_name, layer,
                  failures, errors, skipped, resume_number, cwd)))
        resume_number += 1

    # Now start a few threads at a time.
    running_threads = []
    results_iter = iter(results)
    current_result = next(results_iter)
    last_layer_intermediate_output = None
    output = None
    # Get an object that (only) accepts bytes
    stdout = _get_output_buffer(sys.stdout)
    while ready_threads or running_threads:
        while len(running_threads) < options.processes and ready_threads:
            thread = ready_threads.pop(0)
            thread.start()
            running_threads.append(thread)

        for index, thread in reversed(list(enumerate(running_threads))):
            if not thread.is_alive():
                del running_threads[index]

        # Clear out any messages in queue
        while stdout_queue is not None:
            previous_output = output
            try:
                layer_name, output = stdout_queue.get(False)
            except queue.Empty:
                break
            if layer_name != last_layer_intermediate_output:
                # Clarify what layer is reporting activity.
                if previous_output is not None:
                    stdout.write(b']\n')
                stdout.write(
                    ('[Parallel tests running in '
                     '%s:\n  ' % (layer_name,)).encode('utf-8'))
                last_layer_intermediate_output = layer_name
            if not isinstance(output, bytes):
                output = output.encode('utf-8')
            stdout.write(output)
        # Display results in the order they would have been displayed, had the
        # work not been done in parallel.
        while current_result and current_result.done:
            if output is not None:
                stdout.write(b']\n')
                output = None
            stdout.writelines(current_result.stdout)

            try:
                current_result = next(results_iter)
            except StopIteration:
                current_result = None

        # Help keep-alive monitors (human or automated) keep up-to-date.
        stdout.flush()
        time.sleep(0.01)  # Keep the loop from being too tight.

    # Return the total number of tests run.
    return sum(r.num_ran for r in results)


def tear_down_unneeded(options, needed, setup_layers, errors, optional=False):
    # Tear down any layers not needed for these tests. The unneeded layers
    # might interfere.
    unneeded = [layer for layer in setup_layers if layer not in needed]
    unneeded = order_by_bases(unneeded)
    unneeded.reverse()
    output = options.output
    for layer in unneeded:
        output.start_tear_down(name_from_layer(layer))
        t = time.time()
        try:
            try:
                if hasattr(layer, 'tearDown'):
                    layer.tearDown()
            except NotImplementedError:
                output.tear_down_not_supported()
                if not optional:
                    raise CanNotTearDown(layer)
            except MemoryError:
                raise
            except Exception:
                handle_layer_failure(
                    TearDownLayerFailure(layer), output, errors)
            else:
                output.stop_tear_down(time.time() - t)
        finally:
            del setup_layers[layer]


cant_pm_in_subprocess_message = """
Can't post-mortem debug when running a layer as a subprocess!
Try running layer %r by itself.
"""


def setup_layer(options, layer, setup_layers):
    assert layer is not object
    output = options.output
    if layer not in setup_layers:
        for base in layer.__bases__:
            if base is not object:
                setup_layer(options, base, setup_layers)
        output.start_set_up(name_from_layer(layer))
        t = time.time()
        if hasattr(layer, 'setUp'):
            try:
                layer.setUp()
            except MemoryError:
                raise
            except Exception:
                if options.post_mortem:
                    if options.resume_layer:
                        options.output.error_with_banner(
                            cant_pm_in_subprocess_message
                            % options.resume_layer)
                        raise
                    else:
                        zope.testrunner.debug.post_mortem(
                            sys.exc_info())
                else:
                    raise

        output.stop_set_up(time.time() - t)
        setup_layers[layer] = 1


class TestResult(unittest.TestResult):

    def __init__(self, options, tests, layer_name=None):
        unittest.TestResult.__init__(self)
        self.options = options
        # Calculate our list of relevant layers we need to call testSetUp
        # and testTearDown on.
        layers = []
        gather_layers(layer_from_name(layer_name), layers)
        self.layers = order_by_bases(layers)
        count = 0
        for test in tests:
            count += test.countTestCases()
        self.count = count
        self._stdout_buffer = None
        self._stderr_buffer = None
        self._original_stdout = sys.stdout
        self._original_stderr = sys.stderr

    def testSetUp(self):
        """A layer may define a setup method to be called before each
        individual test.
        """
        for layer in self.layers:
            if hasattr(layer, 'testSetUp'):
                layer.testSetUp()

    def testTearDown(self):
        """A layer may define a teardown method to be called after each
           individual test.

           This is useful for clearing the state of global
           resources or resetting external systems such as relational
           databases or daemons.
        """
        for layer in self.layers[-1::-1]:
            if hasattr(layer, 'testTearDown'):
                layer.testTearDown()

    def _makeBufferedStdStream(self):
        """Make a buffered stream to replace a standard stream."""
        # The returned stream needs to have a 'buffer' attribute, since
        # some tests may expect that to exist on standard streams, and a
        # 'getvalue' method for the convenience of _restoreStdStreams.
        # This requires some care.
        class BufferedStandardStream(io.TextIOWrapper):
            def getvalue(self):
                return self.buffer.getvalue().decode(
                    encoding=self.encoding, errors=self.errors)

        return BufferedStandardStream(
            io.BytesIO(), newline='\n', write_through=True)

    def _setUpStdStreams(self):
        """Set up buffered standard streams, if requested."""
        if self.options.buffer:
            if self._stdout_buffer is None:
                self._stdout_buffer = self._makeBufferedStdStream()
            if self._stderr_buffer is None:
                self._stderr_buffer = self._makeBufferedStdStream()
            sys.stdout = self._stdout_buffer
            sys.stderr = self._stderr_buffer

    def _restoreStdStreams(self):
        """Restore the buffered standard streams and return any contents."""
        if self.options.buffer:
            stdout = sys.stdout.getvalue()
            stderr = sys.stderr.getvalue()
            sys.stdout = self._original_stdout
            sys.stderr = self._original_stderr
            self._stdout_buffer.seek(0)
            self._stdout_buffer.truncate(0)
            self._stderr_buffer.seek(0)
            self._stderr_buffer.truncate(0)
            return stdout, stderr
        else:
            return None, None

    def startTest(self, test):
        self._test_state = test.__dict__.copy()
        self.testSetUp()
        unittest.TestResult.startTest(self, test)
        testsRun = self.testsRun - 1  # subtract the one the base class added
        count = test.countTestCases()
        self.testsRun = testsRun + count

        self.options.output.start_test(test, self.testsRun, self.count)

        self._threads = threadsupport.enumerate()
        self._start_time = time.time()

        self._setUpStdStreams()

    def addSuccess(self, test):
        self._restoreStdStreams()
        t = max(time.time() - self._start_time, 0.0)
        self.options.output.test_success(test, t)

    def addSkip(self, test, reason):
        if not hasattr(self, "_test_state"):
            # ``startTest`` was not called -- set up extected state
            self._test_state = test.__dict__
            count = test.countTestCases()
            self.testsRun += count
            self.options.output.start_test(test, self.testsRun, self.count)
            self._threads = threadsupport.enumerate()
            if not hasattr(self, "_start_time"):
                self._start_time = time.time()
        else:
            self._restoreStdStreams()
        unittest.TestResult.addSkip(self, test, reason)
        self.options.output.test_skipped(test, reason)

    def addSubTest(self, test, subtest, exc_info):
        if exc_info is None:
            return
        stdout, stderr = self._restoreStdStreams()
        outp = self.options.output
        report = (outp.test_failure
                  if issubclass(exc_info[0], test.failureException)
                  else outp.test_error)
        report(subtest, time.time() - self._start_time,
               exc_info, stdout=stdout, stderr=stderr)

        unittest.TestResult.addSubTest(self, test, subtest, exc_info)

        if self.options.post_mortem:
            if self.options.resume_layer:
                self.options.output.error_with_banner(
                    "Can't post-mortem debug when running a layer as"
                    " a subprocess!")
            else:
                zope.testrunner.debug.post_mortem(exc_info)
        elif self.options.stop_on_error:
            self.stop()

    def addError(self, test, exc_info):
        stdout, stderr = self._restoreStdStreams()
        self.options.output.test_error(test, time.time() - self._start_time,
                                       exc_info,
                                       stdout=stdout, stderr=stderr)

        unittest.TestResult.addError(self, test, exc_info)

        if self.options.post_mortem:
            if self.options.resume_layer:
                self.options.output.error_with_banner("Can't post-mortem debug"
                                                      " when running a layer"
                                                      " as a subprocess!")
            else:
                zope.testrunner.debug.post_mortem(exc_info)
        elif self.options.stop_on_error:
            self.stop()

    def addFailure(self, test, exc_info):
        stdout, stderr = self._restoreStdStreams()
        self.options.output.test_failure(test, time.time() - self._start_time,
                                         exc_info,
                                         stdout=stdout, stderr=stderr)

        unittest.TestResult.addFailure(self, test, exc_info)

        if self.options.post_mortem:
            # XXX: mgedmin: why isn't there a resume_layer check here like
            # in addError?
            zope.testrunner.debug.post_mortem(exc_info)
        elif self.options.stop_on_error:
            self.stop()

    def addExpectedFailure(self, test, exc_info):
        self._restoreStdStreams()
        t = max(time.time() - self._start_time, 0.0)
        self.options.output.test_success(test, t)

        unittest.TestResult.addExpectedFailure(self, test, exc_info)

    def addUnexpectedSuccess(self, test):
        stdout, stderr = self._restoreStdStreams()
        self.options.output.test_error(
            test, time.time() - self._start_time,
            (UnexpectedSuccess, UnexpectedSuccess(), None),
            stdout=stdout, stderr=stderr)

        unittest.TestResult.addUnexpectedSuccess(self, test)

        if self.options.post_mortem:
            if self.options.resume_layer:
                self.options.output.error_with_banner("Can't post-mortem debug"
                                                      " when running a layer"
                                                      " as a subprocess!")
            else:
                # XXX: what exc_info? there's no exc_info!
                # flake8 is correct, but keep it quiet for now ...
                zope.testrunner.debug.post_mortem(exc_info)  # noqa: F821
        elif self.options.stop_on_error:
            self.stop()

    def stopTest(self, test):
        self.testTearDown()
        # Without clearing, cyclic garbage referenced by the test
        # would be reported in the following test.
        test.__dict__.clear()
        test.__dict__.update(self._test_state)
        del self._test_state
        cycles = None
        if (uses_refcounts and self.options.gc_after_test and
                self.options.verbose >= 4):
            gc_opts = gc.get_debug()
            gc.set_debug(gc.DEBUG_SAVEALL)
            gc.collect()
            if gc.garbage:
                g = DiGraph(gc.garbage)
                for obj in gc.garbage:
                    g.add_neighbors(obj, gc.get_referents(obj))
                cycles = [[repr_lines(o) for o in c] for c in g.sccs()]
                del gc.garbage[:]
                g = obj = None  # avoid to hold cyclic garbage
            gc.set_debug(gc_opts)
        gccount = gc.collect() \
            if uses_refcounts and self.options.gc_after_test else 0
        self.options.output.stop_test(test, gccount)

        if cycles:
            self.options.output.test_cycles(test, cycles)

        if is_jython:
            pass
        else:
            if gc.garbage:
                self.options.output.test_garbage(test, gc.garbage)
                # TODO: Perhaps eat the garbage here, so that the garbage isn't
                #       printed for every subsequent test.

        # Did the test leave any new threads behind?
        new_threads = []
        for t in threadsupport.enumerate():
            if t.is_alive() and t not in self._threads:
                if not any([re.match(p, t.name)
                            for p in self.options.ignore_new_threads]):
                    new_threads.append(t)

        if new_threads:
            self.options.output.test_threads(test, new_threads)


def layer_from_name(layer_name):
    """Return the layer for the corresponding layer_name by discovering
       and importing the necessary module if necessary.

       Note that a name -> layer cache is maintained by name_from_layer
       to allow locating layers in cases where it would otherwise be
       impossible.
    """
    if layer_name in _layer_name_cache:
        return _layer_name_cache[layer_name]
    layer_names = layer_name.split('.')
    layer_module, module_layer_name = layer_names[:-1], layer_names[-1]
    module_name = '.'.join(layer_module)
    module = import_name(module_name)
    try:
        return getattr(module, module_layer_name)
    except AttributeError:
        # the default error is very uninformative:
        #   AttributeError: 'module' object has no attribute 'DemoLayer'
        # it doesn't say *which* module
        raise AttributeError('module %r has no attribute %r'
                             % (module_name, module_layer_name))


def layer_sort_key(layer):
    """Compute sort key for layers.

    Based on the reverse MRO ordering in order to put layers with shared base
    layers next to each other.
    """
    seen = set()
    key = []

    # Note: we cannot reuse gather_layers() here because it uses a
    # different traversal order.
    binding = {}  # hack to avoid recursion -- for PY2

    def _gather(layer):
        seen.add(layer)
        # We make the simplifying assumption that the order of initialization
        # of base layers does not matter.  Given that, traversing the bases
        # in reverse order here keeps the ordering of layers in
        # testrunner-layers.rst the same as it was in older versions of
        # zope.testrunner, so let's use that.
        for base in layer.__bases__[::-1]:
            if base is not object and base not in seen:
                binding["_gather"](base)
        key.append(layer)

    binding["_gather"] = _gather
    _gather(layer)
    try:
        return tuple(name_from_layer(ly) for ly in key if ly != UnitTests)
    finally:
        binding.clear()  # break reference cycle


def order_by_bases(layers):
    """Order the layers from least to most specific (bottom to top).

    Puts unit tests first.  Groups layers with common base layers together.
    Sorts the rest alphabetically.  Removes duplicates.
    """
    layers = sorted(layers, key=layer_sort_key, reverse=True)
    gathered = []
    for layer in layers:
        gather_layers(layer, gathered)
    gathered.reverse()
    seen = {}
    result = []
    for layer in gathered:
        if layer not in seen:
            seen[layer] = 1
            if layer in layers:
                result.append(layer)
    return result


def gather_layers(layer, result):
    if layer is not object:
        result.append(layer)
    for b in layer.__bases__:
        gather_layers(b, result)


class FakeInputContinueGenerator:

    def readline(self):
        print('c\n')
        print('*' * 70)
        print("Can't use pdb.set_trace when running a layer"
              " as a subprocess!")
        print('*' * 70)
        print()
        return 'c\n'

    def close(self):
        pass


def repr_lines(obj, max_width=75, max_lines=5):
    """represent *obj* by a sequence of text lines.

    Use at most *max_lines*, each with at most *max_width* chars.
    """
    try:
        oi = pprint.pformat(obj)
    except Exception:
        # unprintable
        oi = f"{obj.__class__} instance at 0x{id(obj):x}"
    # limit
    cmps = oi.split("\n", max_lines)
    if len(cmps) > max_lines:
        cmps[-1] = "..."
    for i, li in enumerate(cmps):
        if len(li) > max_width:
            cmps[i] = li[:max_width - 3] + "..."
    return cmps



from unittest import TestCase

from ..digraph import DiGraph


class Tests(TestCase):
    def test_int_digraph(self):
        self.check(lambda nodes: DiGraph(nodes, False))

    def test_obj_diraph(self):
        self.check(DiGraph)

    def check(self, digraph_factory):
        ints = list(range(100))
        g = digraph_factory(ints)
        self.assertEqual(sorted(g.nodes()), ints)
        nb0 = [ints[0], ints[1]]
        g.add_neighbors(ints[0], nb0)
        self.assertEqual(sorted(g.neighbors(ints[0])), nb0)
        self.assertEqual(list(g.neighbors(ints[1])), [])
        # unknown neighbors
        with self.assertRaises(KeyError):
            g.add_neighbors(ints[0], (200,), False)
        g.add_neighbors(ints[0], (200,))  # ignored
        self.assertEqual(sorted(g.neighbors(ints[0])), nb0)
        # unknown node
        with self.assertRaises(KeyError):
            g.add_neighbors(200, (ints[0],), False)
        g.add_neighbors(200, (ints[0],))  # ignored

        # additional neighbors
        nb0.append(ints[2])
        g.add_neighbors(ints[0], nb0)
        self.assertEqual(sorted(g.neighbors(ints[0])), nb0)

    def test_scc_linear(self):
        ints = list(range(100))
        g = DiGraph(ints)
        for i in range(99):
            g.add_neighbors(ints[i], (ints[i + 1], ))
        self.assertEqual(sorted(g.sccs(True)), [[i] for i in ints])

    def test_trivial_cycle(self):
        g = dig_from_dict({1: 2, 2: (2, 3), 3: ()})
        sccs = list(g.sccs())
        self.assertEqual(len(sccs), 1)
        cy = sccs[0]
        self.assertEqual(len(cy), 1)
        self.assertEqual(cy[0], 2)

    def test_forest(self):
        g = dig_from_dict({1: (), 2: ()})
        sccs = list(g.sccs(True))
        self.assertEqual(sorted(sccs), [[1], [2]])

    def test_complex(self):
        g = dig_from_dict({0: 1, 1: 2, 2: (3, 4), 3: (0, 4),
                           4: (5, 2), 5: (6, 9), 6: (5, 7),
                           7: 8, 8: 9, 9: (5, 6)})
        sccs = list(g.sccs())
        self.assertEqual(len(sccs), 2)
        for c in sccs:
            self.assertEqual(len(c), 5)
            self.assertEqual(max(c) - min(c), 4)
        self.assertEqual(sorted(sccs[0] + sccs[1]), list(range(10)))


def dig_from_dict(d):
    g = DiGraph(list(d), False)
    for e in d.items():
        n, nbs = e
        if not hasattr(nbs, "__len__"):
            nbs = nbs,
        g.add_neighbors(n, nbs, False)
    return g



##############################################################################
#
# Copyright (c) 2004-2008 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Test runner statistics
"""

import time

import zope.testrunner.feature


class Statistics(zope.testrunner.feature.Feature):

    active = True
    layers_run = 0
    tests_run = 0

    def late_setup(self):
        self.start_time = time.time()

    def early_teardown(self):
        self.end_time = time.time()

    def global_teardown(self):
        self.total_time = self.end_time - self.start_time

    def layer_setup(self, layer):
        self.layers_run += 1

    def report(self):
        if not self.runner.do_run_tests:
            return
        if self.layers_run == 1:
            return
        self.runner.options.output.totals(
            n_tests=self.runner.ran,
            n_failures=len(self.runner.failures),
            n_errors=len(self.runner.errors) + len(self.runner.import_errors),
            n_seconds=self.total_time,
            n_skipped=len(self.runner.skipped))



##############################################################################
#
# Copyright (c) 2010 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Unit tests for the testrunner's filtering functions
"""

import unittest

from zope.testrunner import filter


class TestFilterMakerFunction(unittest.TestCase):
    def test_no_filters(self):
        accept = filter.build_filtering_func([])
        self.assertFalse(accept('test_xx'))
        self.assertFalse(accept('test_yy'))
        self.assertFalse(accept('test_zz'))

    def test_match_all_filter(self):
        accept = filter.build_filtering_func(['.'])
        self.assertTrue(accept('test_xx'))
        self.assertTrue(accept('test_yy'))
        self.assertTrue(accept('test_zz'))

    def test_select_specific_pattern(self):
        accept = filter.build_filtering_func(['xx'])
        self.assertTrue(accept('test_xx'))
        self.assertFalse(accept('test_yy'))
        self.assertFalse(accept('test_zz'))

    def test_select_several_patterns(self):
        accept = filter.build_filtering_func(['xx', 'yy'])
        self.assertTrue(accept('test_xx'))
        self.assertTrue(accept('test_yy'))
        self.assertFalse(accept('test_zz'))

    def test_reject_one_pattern(self):
        accept = filter.build_filtering_func(['!xx'])
        self.assertFalse(accept('test_xx'))
        self.assertTrue(accept('test_yy'))
        self.assertTrue(accept('test_zz'))

    def test_reject_several_patterns(self):
        accept = filter.build_filtering_func(['!xx', '!zz'])
        self.assertFalse(accept('test_xx'))
        self.assertTrue(accept('test_yy'))
        self.assertFalse(accept('test_zz'))

    def test_accept_and_reject(self):
        accept = filter.build_filtering_func(['!xx', 'yy'])
        self.assertFalse(accept('test_xx'))
        self.assertTrue(accept('test_yy'))
        self.assertFalse(accept('test_zz'))

    def test_accept_and_reject_overlap(self):
        accept = filter.build_filtering_func(['!test_', 'yy'])
        self.assertFalse(accept('test_xx'))
        self.assertFalse(accept('test_yy'))
        self.assertFalse(accept('test_zz'))

    def test_accept_and_reject_same(self):
        accept = filter.build_filtering_func(['yy', '!yy'])
        self.assertFalse(accept('test_xx'))
        self.assertFalse(accept('test_yy'))
        self.assertFalse(accept('test_zz'))



##############################################################################
#
# Copyright (c) 2004-2008 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Filter which tests to run.
"""

import zope.testrunner.feature


class Listing(zope.testrunner.feature.Feature):
    """Lists all tests in the report instead of running the tests."""

    def __init__(self, runner):
        super().__init__(runner)
        self.active = bool(runner.options.list_tests)

    def global_setup(self):
        self.runner.do_run_tests = False
        self.runner.failed = False

    def report(self):
        for layer_name, layer, tests in self.runner.ordered_layers():
            self.runner.options.output.list_of_tests(tests, layer_name)



##############################################################################
#
# Copyright (c) 2003 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Sample tests with sleep and layers that can't be torn down
"""

import time
import unittest


class Layer1:

    def setUp(self):
        pass
    setUp = classmethod(setUp)

    def tearDown(self):
        raise NotImplementedError
    tearDown = classmethod(tearDown)


class Layer2:

    def setUp(self):
        pass
    setUp = classmethod(setUp)

    def tearDown(self):
        raise NotImplementedError
    tearDown = classmethod(tearDown)


class TestSomething1(unittest.TestCase):

    layer = Layer1

    def test_something(self):
        pass


class TestSomething2(unittest.TestCase):

    layer = Layer2

    def test_something(self):
        time.sleep(0.5)

    def test_something2(self):
        time.sleep(0.5)


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(
        unittest.defaultTestLoader.loadTestsFromTestCase(TestSomething1))
    suite.addTest(
        unittest.defaultTestLoader.loadTestsFromTestCase(TestSomething2))
    return suite


if __name__ == '__main__':
    unittest.main()



##############################################################################
#
# Copyright (c) 2004 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Sample test layers
"""

layer = '0'  # Internal to samples. Not part of layer API
layerx = '0'


class Layer1:
    # Internal to samples. Not part of layer API:
    layer = '1'
    base = '0'
    layerx = '0'

    def setUp(self):
        global layer
        if layer != self.base:
            raise ValueError(f"Bad layer, {layer}, for {self}.")
        layer = self.layer
    setUp = classmethod(setUp)

    def tearDown(self):
        global layer
        if layer != self.layer:
            raise ValueError(f"Bad layer, {layer}, for {self}.")
        layer = self.base
    tearDown = classmethod(tearDown)


class Layerx:
    layerx = '1'  # Internal to samples. Not part of layer API
    basex = '0'

    def setUp(self):
        global layerx
        if layerx != self.basex:
            raise ValueError(f"Bad layerx, {layerx}, for {self}.")
        layerx = self.layerx
    setUp = classmethod(setUp)

    def tearDown(self):
        global layerx
        if layerx != self.layerx:
            raise ValueError(f"Bad layerx, {layerx}, for {self}.")
        layerx = self.basex
    tearDown = classmethod(tearDown)


class Layer11(Layer1):
    layer = '11'  # Internal to samples. Not part of layer API
    base = '1'    # Internal to samples. Not part of layer API


class Layer12(Layer1):
    layer = '12'  # Internal to samples. Not part of layer API
    base = '1'    # Internal to samples. Not part of layer API


class Layer111(Layerx, Layer11):
    layer = '111'  # Internal to samples. Not part of layer API
    base = '11'    # Internal to samples. Not part of layer API
    layerx = '2'   # Internal to samples. Not part of layer API
    basex = '1'

    def setUp(self):
        global layer
        if layer != self.base:
            raise ValueError(f"Bad layer, {layer}, for {self}.")
        layer = self.layer
        global layerx
        if layerx != self.basex:
            raise ValueError(f"Bad layerx, {layerx}, for {self}.")
        layerx = self.layerx
    setUp = classmethod(setUp)

    def tearDown(self):
        global layer
        if layer != self.layer:
            raise ValueError(f"Bad layer, {layer}, for {self}.")
        layer = self.base
        global layerx
        if layerx != self.layerx:
            raise ValueError(f"Bad layerx, {layerx}, for {self}.")
        layerx = self.basex
    tearDown = classmethod(tearDown)


class Layer121(Layer12):
    layer = '121'  # Internal to samples. Not part of layer API
    base = '12'    # Internal to samples. Not part of layer API


class Layer112(Layerx, Layer11):
    layer = '112'  # Internal to samples. Not part of layer API
    base = '11'    # Internal to samples. Not part of layer API
    layerx = '2'   # Internal to samples. Not part of layer API
    basex = '1'

    def setUp(self):
        global layer
        if layer != self.base:
            raise ValueError(f"Bad layer, {layer}, for {self}.")
        layer = self.layer
        global layerx
        if layerx != self.basex:
            raise ValueError(f"Bad layerx, {layerx}, for {self}.")
        layerx = self.layerx
    setUp = classmethod(setUp)

    def tearDown(self):
        global layer
        if layer != self.layer:
            raise ValueError(f"Bad layer, {layer}, for {self}.")
        layer = self.base
        global layerx
        if layerx != self.layerx:
            raise ValueError(f"Bad layerx, {layerx}, for {self}.")
        layerx = self.basex
    tearDown = classmethod(tearDown)


class Layer122(Layer12):
    layer = '122'  # Internal to samples. Not part of layer API
    base = '12'    # Internal to samples. Not part of layer API



##############################################################################
#
# Copyright (c) 2004-2008 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Support for tracking reference counts.
"""

import gc
import sys
import types


class TrackRefs:
    """Object to track reference counts across test runs."""

    def __init__(self):
        self.type2count = {}
        self.type2all = {}
        self.delta = None
        self.n = 0
        self.update()
        self.delta = None

    def update(self):
        gc.collect()
        obs = sys.getobjects(0)
        type2count = {}
        type2all = {}
        n = 0
        for o in obs:
            if type(o) is str and o == '<dummy key>':
                # avoid dictionary madness
                continue

            all = sys.getrefcount(o) - 3
            n += all

            t = type(o)
            if issubclass(t, types.InstanceType):
                t = o.__class__

            if t in type2count:
                type2count[t] += 1
                type2all[t] += all
            else:
                type2count[t] = 1
                type2all[t] = all

        ct = [(
            type_or_class_title(t),
            type2count[t] - self.type2count.get(t, 0),
            type2all[t] - self.type2all.get(t, 0),
        )
            for t in type2count]
        ct += [(
            type_or_class_title(t),
            - self.type2count[t],
            - self.type2all[t],
        )
            for t in self.type2count
            if t not in type2count]
        ct.sort()
        self.delta = ct
        self.type2count = type2count
        self.type2all = type2all
        self.n = n

    def output(self):
        printed = False
        s1 = s2 = 0
        for t, delta1, delta2 in self.delta:
            if delta1 or delta2:
                if not printed:
                    print(
                        '    Leak details, changes in instances and refcounts'
                        ' by type/class:')
                    print(
                        "    %-55s %6s %6s" % ('type/class', 'insts', 'refs'))
                    print("    %-55s %6s %6s" % ('-' * 55, '-----', '----'))
                    printed = True
                print("    %-55s %6d %6d" % (t, delta1, delta2))
                s1 += delta1
                s2 += delta2

        if printed:
            print("    %-55s %6s %6s" % ('-' * 55, '-----', '----'))
            print("    %-55s %6s %6s" % ('total', s1, s2))

        self.delta = None


def type_or_class_title(t):
    module = getattr(t, '__module__', '__builtin__')
    if module == '__builtin__':
        return t.__name__
    return f"{module}.{t.__name__}"



##############################################################################
#
# Copyright (c) 2003 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Sample tests with a layer that can't be torn down
"""

import doctest
import unittest


class Layer:

    def setUp(self):
        pass
    setUp = classmethod(setUp)

    def tearDown(self):
        raise NotImplementedError
    tearDown = classmethod(tearDown)


class TestSomething(unittest.TestCase):

    layer = Layer

    def test_something(self):
        import pdb  # noqa: T100 import for pdb found
        pdb.set_trace()  # noqa: T100 pdb.set_trace found

    def test_something2(self):
        import pdb  # noqa: T100 import for pdb found
        pdb.set_trace()  # noqa: T100 pdb.set_trace found

    def test_something3(self):
        import pdb  # noqa: T100 import for pdb found
        pdb.set_trace()  # noqa: T100 pdb.set_trace found

    def test_something4(self):
        import pdb  # noqa: T100 import for pdb found
        pdb.set_trace()  # noqa: T100 pdb.set_trace found

    def test_something5(self):
        f()


def f():
    import pdb  # noqa: T100 import for pdb found
    pdb.set_trace()  # noqa: T100 pdb.set_trace found


def test_set_trace():
    """
    >>> if 1:
    ...     x = 1
    ...     import pdb; pdb.set_trace()
    """


def test_set_trace2():
    """
    >>> f()
    """


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(
        unittest.defaultTestLoader.loadTestsFromTestCase(TestSomething))
    d = doctest.DocTestSuite()
    d.layer = Layer
    suite.addTest(d)
    return suite


if __name__ == '__main__':
    unittest.main()



##############################################################################
#
# Copyright (c) 2003 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

import doctest
import unittest

import samplelayers


layername = 'samplelayers.Layer121'
layer = samplelayers.Layer121

x = 0
y = 0
z = 0


class TestA(unittest.TestCase):
    layer = layername

    def setUp(self):
        global x
        x = 1

    def tearDown(self):
        global x
        x = 0

    def test_x1(self):
        self.assertEqual(x, 1)
        self.assertEqual(samplelayers.layer, layer.layer)
        self.assertEqual(samplelayers.layerx, layer.layerx)

    def test_y0(self):
        self.assertEqual(y, 0)
        self.assertEqual(samplelayers.layer, layer.layer)
        self.assertEqual(samplelayers.layerx, layer.layerx)

    def test_z0(self):
        self.assertEqual(z, 0)
        self.assertEqual(samplelayers.layer, layer.layer)
        self.assertEqual(samplelayers.layerx, layer.layerx)


class TestB(unittest.TestCase):
    layer = layername

    def setUp(self):
        global y
        y = 1

    def tearDown(self):
        global y
        y = 0

    def test_y1(self):
        self.assertEqual(y, 1)
        self.assertEqual(samplelayers.layer, layer.layer)
        self.assertEqual(samplelayers.layerx, layer.layerx)

    def test_x0(self):
        self.assertEqual(x, 0)
        self.assertEqual(samplelayers.layer, layer.layer)
        self.assertEqual(samplelayers.layerx, layer.layerx)

    def test_z0(self):
        self.assertEqual(z, 0)
        self.assertEqual(samplelayers.layer, layer.layer)
        self.assertEqual(samplelayers.layerx, layer.layerx)


class TestNotMuch(unittest.TestCase):
    layer = layername

    def test_1(self):
        self.assertEqual(samplelayers.layer, layer.layer)
        self.assertEqual(samplelayers.layerx, layer.layerx)

    def test_2(self):
        self.assertEqual(samplelayers.layer, layer.layer)
        self.assertEqual(samplelayers.layerx, layer.layerx)

    def test_3(self):
        self.assertEqual(samplelayers.layer, layer.layer)
        self.assertEqual(samplelayers.layerx, layer.layerx)


def setUp(test):
    test.globs['z'] = 1
    test.globs['layer'] = layer.layer
    test.globs['layerx'] = layer.layerx
    test.globs['samplelayers'] = samplelayers


def test_y0(self):
    """
    >>> y = 0
    >>> y
    0

    >>> (layer == samplelayers.layer), (layerx == samplelayers.layerx)
    (True, True)
    """


def test_x0(self):
    """
    >>> x = 0
    >>> x
    0

    >>> (layer == samplelayers.layer), (layerx == samplelayers.layerx)
    (True, True)
    """


def test_z1(self):
    """
    >>> z
    1

    >>> (layer == samplelayers.layer), (layerx == samplelayers.layerx)
    (True, True)
    """


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(TestA))
    suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(TestB))
    suite.addTest(
        unittest.defaultTestLoader.loadTestsFromTestCase(TestNotMuch))
    s = doctest.DocTestSuite(setUp=setUp)
    s.layer = layer
    suite.addTest(s)
    s = doctest.DocFileSuite('../../sampletestsl.rst', setUp=setUp)
    s.layer = layer
    suite.addTest(s)
    return suite



##############################################################################
#
# Copyright (c) 2004-2008 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Profiler support for the test runner
"""
import cProfile
import glob
import os
import pstats
import tempfile

import zope.testrunner.feature


available_profilers = {}


class CProfiler:
    """cProfiler"""

    def __init__(self, filepath):
        self.filepath = filepath
        self.profiler = cProfile.Profile()
        self.enable = self.profiler.enable
        self.disable = self.profiler.disable

    def finish(self):
        self.profiler.dump_stats(self.filepath)

    def loadStats(self, prof_glob):
        stats = None
        for file_name in glob.glob(prof_glob):
            if stats is None:
                stats = pstats.Stats(file_name)
            else:
                stats.add(file_name)
        return stats


available_profilers['cProfile'] = CProfiler


class Profiling(zope.testrunner.feature.Feature):

    def __init__(self, runner):
        super().__init__(runner)
        self.active = bool(self.runner.options.profile)
        self.profiler = self.runner.options.profile

    def global_setup(self):
        self.prof_prefix = 'tests_profile.'
        self.prof_suffix = '.prof'
        self.prof_glob = os.path.join(
            self.runner.options.prof_dir,
            self.prof_prefix + '*' + self.prof_suffix)
        # if we are going to be profiling, and this isn't a subprocess,
        # clean up any stale results files
        if not self.runner.options.resume_layer:
            for file_name in glob.glob(self.prof_glob):
                os.unlink(file_name)
        # set up the output file
        self.oshandle, self.file_path = tempfile.mkstemp(
            self.prof_suffix, self.prof_prefix, self.runner.options.prof_dir)
        self.profiler = available_profilers[self.runner.options.profile](
            self.file_path)

        # Need to do this rebinding to support the stack-frame annoyance with
        # hotshot.
        self.late_setup = self.profiler.enable
        self.early_teardown = self.profiler.disable

    def global_teardown(self):
        self.profiler.finish()
        # We must explicitly close the handle mkstemp returned, else on
        # Windows this dies the next time around just above due to an
        # attempt to unlink a still-open file.
        os.close(self.oshandle)
        if not self.runner.options.resume_layer:
            self.profiler_stats = self.profiler.loadStats(self.prof_glob)
            self.profiler_stats.sort_stats('cumulative', 'calls')

    def report(self):
        if not self.runner.options.resume_layer:
            self.runner.options.output.profiler_stats(self.profiler_stats)



##############################################################################
#
# Copyright (c) 2004-2008 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Garbage collection support.
"""

import gc
import sys

import zope.testrunner.feature


class Threshold(zope.testrunner.feature.Feature):

    def __init__(self, runner):
        super().__init__(runner)
        self.threshold = self.runner.options.gc
        self.active = bool(self.threshold)

        if not self.active:
            return

        if len(self.threshold) > 3:
            self.runner.options.output.error("Too many --gc options")
            sys.exit(1)

    def global_setup(self):
        self.old_threshold = gc.get_threshold()

        if self.threshold[0]:
            self.runner.options.output.info(
                "Cyclic garbage collection threshold set to: %s" %
                repr(tuple(self.threshold)))
        else:
            self.runner.options.output.info(
                "Cyclic garbage collection is disabled.")

        gc.set_threshold(*self.threshold)

    def global_teardown(self):
        gc.set_threshold(*self.old_threshold)


class Debug(zope.testrunner.feature.Feature):
    """Manages garbage collection debug flags."""

    def __init__(self, runner):
        super().__init__(runner)
        self.flags = self.runner.options.gc_option
        self.active = bool(self.flags)

        if not self.active:
            return

    def global_setup(self):
        # Set garbage collection debug flags
        self.old_flags = gc.get_debug()
        new_flags = 0
        for op in self.flags:
            new_flags |= getattr(gc, op)
        gc.set_debug(new_flags)

    def global_teardown(self):
        gc.set_debug(self.old_flags)



##############################################################################
#
# Copyright (c) 2022 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Thread support beyond ``threading``.

``threading.enumerate`` may or may not know threads
started with ``_thread.start_new_thread``. If it knows them,
it does not know when they stop.
If ``sys._current_frames`` is available, use this to
reliable determine the currently running threads.
"""
import sys
import threading


current_frames = getattr(sys, "_current_frames", None)

if current_frames is None:  # pragma: no cover
    enumerate = threading.enumerate
else:
    def enumerate():
        """return sequence of proxies for the currently running threads."""
        running = set(current_frames())
        th_known = {t.ident: t for t in threading.enumerate()}
        return [ThreadProxy(th_known[i] if i in th_known else DummyThread(i))
                for i in running]


class ThreadProxy:
    """auxiliary class to provide ident based ``__eq__``."""

    def __init__(self, thread):
        self.thread = thread

    def __eq__(self, other):
        return self.thread.ident == other.thread.ident

    def __repr__(self):
        return repr(self.thread)

    def __getattr__(self, k):
        return getattr(self.thread, k)


class DummyThread:
    """auxiliary to represent a thread unknown to ``threading``."""

    def __init__(self, ident):
        self.ident = ident
        self.name = "Dummy-%s" % ident

    def __repr__(self):
        return "DummyThread %s, started, daemon" % self.ident

    def is_alive(self):
        return True



##############################################################################
#
# Copyright (c) 2004, 2013 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
# This package is developed by the Zope Toolkit project, documented here:
# http://docs.zope.org/zopetoolkit
# When developing and releasing this package, please follow the documented
# Zope Toolkit policies as described by this documentation.
##############################################################################
import os

from setuptools import setup


version = '7.2'

INSTALL_REQUIRES = [
    'setuptools',
    'zope.exceptions',
    'zope.interface',
]

TESTS_REQUIRE = [
    'zope.testing',
]

EXTRAS_REQUIRE = {
    'test': TESTS_REQUIRE,
    'subunit': [
        'python-subunit >= 1.4.3',
        'testtools >= 0.9.30',
    ],
    'docs': [
        'Sphinx',
        'sphinxcontrib-programoutput',
    ],
}


def read(*names):
    with open(os.path.join(*names)) as f:
        return f.read()


long_description = (
    read('README.rst')
    + '\n\n' +
    read("docs", 'getting-started.rst')
    + '\n\n' +
    read('CHANGES.rst')
)

setup(
    name='zope.testrunner',
    version=version,
    url='https://github.com/zopefoundation/zope.testrunner',
    license='ZPL-2.1',
    description='Zope testrunner script.',
    long_description=long_description,
    author='Zope Foundation and Contributors',
    author_email='zope-dev@zope.dev',
    packages=[
        "zope",
        "zope.testrunner",
    ],
    package_dir={'': 'src'},
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Framework :: Zope :: 3",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Zope Public License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: 3.13",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Software Development :: Testing",
    ],
    namespace_packages=['zope'],
    python_requires='>=3.9',
    install_requires=INSTALL_REQUIRES,
    extras_require=EXTRAS_REQUIRE,
    entry_points={
        'console_scripts':
            ['zope-testrunner = zope.testrunner:run'],
    },
    include_package_data=True,
    zip_safe=False,
)



##############################################################################
#
# Copyright (c) 2003 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

"""Sample tests that produce output on stdout and stderr."""

import sys
import unittest


class Test(unittest.TestCase):

    def _getStreamBuffer(self, stream):
        return stream.buffer

    def test_stdout_success(self):
        sys.stdout.write("stdout output on success\n")
        self._getStreamBuffer(sys.stdout).write(
            b"stdout buffer output on success\n")

    def test_stdout_failure(self):
        sys.stdout.write("stdout output on failure\n")
        self._getStreamBuffer(sys.stdout).write(
            b"stdout buffer output on failure\n")
        self.assertTrue(False)

    def test_stdout_error(self):
        sys.stdout.write("stdout output on error\n")
        self._getStreamBuffer(sys.stdout).write(
            b"stdout buffer output on error\n")
        raise Exception("boom")

    def test_stderr_success(self):
        sys.stderr.write("stderr output on success\n")
        self._getStreamBuffer(sys.stderr).write(
            b"stderr buffer output on success\n")

    def test_stderr_failure(self):
        sys.stderr.write("stderr output on failure\n")
        self._getStreamBuffer(sys.stderr).write(
            b"stderr buffer output on failure\n")
        self.assertTrue(False)

    def test_stderr_error(self):
        sys.stderr.write("stderr output on error\n")
        self._getStreamBuffer(sys.stderr).write(
            b"stderr buffer output on error\n")
        raise Exception("boom")


def test_suite():
    return unittest.defaultTestLoader.loadTestsFromTestCase(Test)



##############################################################################
#
# Copyright (c) 2004-2008 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Code coverage analysis
"""

import os.path
import sys
import threading
import trace

import zope.testrunner.feature
from zope.testrunner.find import test_dirs


# For some reason, the doctest module resets the trace callable randomly, thus
# disabling the coverage. Simply disallow the code from doing this. A real
# trace can be set, so that debugging still works.
osettrace = sys.settrace


def settrace(trace):
    if trace is None:  # pragma: no cover
        return
    osettrace(trace)


class TestTrace(trace.Trace):
    """Simple tracer.

    >>> tracer = TestTrace([], count=False, trace=False)

    Simple rules for use: you can't stop the tracer if it not started
    and you can't start the tracer if it already started:

    >>> tracer.stop()
    Traceback (most recent call last):
        File 'testrunner.py'
    AssertionError: can't stop if not started

    >>> tracer.start()
    >>> tracer.start()
    Traceback (most recent call last):
        File 'testrunner.py'
    AssertionError: can't start if already started

    >>> tracer.stop()
    >>> tracer.stop()
    Traceback (most recent call last):
        File 'testrunner.py'
    AssertionError: can't stop if not started
    """

    def __init__(self, directories, **kw):
        trace.Trace.__init__(self, **kw)
        self.ignore = TestIgnore(directories)
        self.started = False

    def start(self):
        assert not self.started, "can't start if already started"
        if not self.donothing:
            sys.settrace = settrace
            sys.settrace(self.globaltrace)
            threading.settrace(self.globaltrace)
        self.started = True

    def stop(self):
        assert self.started, "can't stop if not started"
        if not self.donothing:
            sys.settrace = osettrace
            sys.settrace(None)
            threading.settrace(None)
        self.started = False


class TestIgnore:

    def __init__(self, directories):
        self._test_dirs = [self._filenameFormat(d[0]) + os.path.sep
                           for d in directories]
        self._ignore = {}
        self._ignored = self._ignore.get

    def names(self, filename, modulename):
        # Special case: Modules generated from text files; i.e. doctests
        if modulename == '<string>':
            return True
        filename = self._filenameFormat(filename)
        ignore = self._ignored(filename)
        if ignore is None:
            ignore = True
            if filename is not None:
                for d in self._test_dirs:
                    if filename.startswith(d):
                        ignore = False
                        break
            self._ignore[filename] = ignore
        return ignore

    def _filenameFormat(self, filename):
        return os.path.abspath(filename)


if sys.platform == 'win32':
    # on win32 drive name can be passed with different case to `names`
    # that lets e.g. the coverage profiler skip complete files
    # _filenameFormat will make sure that all drive and filenames get
    # lowercased albeit trace coverage has still problems with lowercase
    # drive letters when determining the dotted module name
    OldTestIgnore = TestIgnore

    class TestIgnore(OldTestIgnore):
        def _filenameFormat(self, filename):
            return os.path.normcase(os.path.abspath(filename))


class Coverage(zope.testrunner.feature.Feature):

    tracer = None
    directory = None

    def __init__(self, runner):
        super().__init__(runner)
        self.active = bool(runner.options.coverage)

    def global_setup(self):
        """Executed once when the test runner is being set up."""
        self.directory = os.path.join(
            os.getcwd(), self.runner.options.coverage)

        # FIXME: This shouldn't rely on the find feature directly.
        self.tracer = TestTrace(test_dirs(self.runner.options, {}),
                                trace=False, count=True)
        self.tracer.start()

    def early_teardown(self):
        """Executed once directly after all tests."""
        self.tracer.stop()

    def report(self):
        """Executed once after all tests have been run and all setup was
        torn down."""
        r = self.tracer.results()
        r.write_results(summary=True, coverdir=self.directory)



##############################################################################
#
# Copyright (c) 2015 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Unit tests for the testrunner's runner logic
"""
import sys
import unittest

from zope.testrunner import runner
from zope.testrunner.layer import UnitTests


class TestLayerOrdering(unittest.TestCase):

    def sort_key(self, layer):
        return ', '.join(qn.rpartition('.')[-1]
                         for qn in runner.layer_sort_key(layer))

    def order(self, *layers):
        return ', '.join(layer.__name__
                         for layer in runner.order_by_bases(layers))

    def test_order_by_bases(self):
        #    A      B
        #   /|\    /
        #  / | \  /
        # A1 A2 AB
        class A:
            pass

        class A1(A):
            pass

        class A2(A):
            pass

        class B:
            pass

        class AB(A, B):
            pass
        self.assertEqual(self.order(B, A1, A2, A1, AB, UnitTests),
                         'UnitTests, A1, A2, B, AB')
        self.assertEqual(self.sort_key(UnitTests), '')
        self.assertEqual(self.sort_key(A1), 'A, A1')
        self.assertEqual(self.sort_key(A2), 'A, A2')
        self.assertEqual(self.sort_key(B), 'B')
        self.assertEqual(self.sort_key(AB), 'B, A, AB')

    def test_order_by_bases_alphabetical_order(self):
        class X:
            pass

        class Y:
            pass

        class Z:
            pass

        class A(Y):
            pass

        class B(X):
            pass

        class C(Z):
            pass
        # It'd be nice to get A, B, C here, but that's not trivial.
        self.assertEqual(self.order(A, B, C), 'B, A, C')
        self.assertEqual(self.sort_key(B), 'X, B')
        self.assertEqual(self.sort_key(A), 'Y, A')
        self.assertEqual(self.sort_key(C), 'Z, C')

    def test_order_by_bases_diamond_hierarchy(self):
        #      A
        #     / \
        #    B   D
        #    |   |
        #    C   E
        #     \ /
        #      F
        class A:
            pass

        class B(A):
            pass

        class C(B):
            pass

        class D(A):
            pass

        class E(D):
            pass

        class F(C, E):
            pass
        self.assertEqual(self.order(A, B, C, D, E, F), 'A, B, C, D, E, F')
        self.assertEqual(self.sort_key(A), 'A')
        self.assertEqual(self.sort_key(B), 'A, B')
        self.assertEqual(self.sort_key(C), 'A, B, C')
        self.assertEqual(self.sort_key(D), 'A, D')
        self.assertEqual(self.sort_key(E), 'A, D, E')
        self.assertEqual(self.sort_key(F), 'A, D, E, B, C, F')
        # only those layers that were passed in are returned
        self.assertEqual(self.order(B, E), 'B, E')

    def test_order_by_bases_shared_setup_trumps_alphabetical_order(self):
        #       A
        #      / \
        #     AB AC
        #    /  \  \
        # AAAABD \ MMMACF
        #      ZZZABE
        class A:
            pass

        class AB(A):
            pass

        class AC(A):
            pass

        class AAAABD(AB):
            pass

        class ZZZABE(AB):
            pass

        class MMMACF(AC):
            pass
        self.assertEqual(self.order(AAAABD, MMMACF, ZZZABE),
                         'AAAABD, ZZZABE, MMMACF')
        self.assertEqual(self.sort_key(AAAABD), 'A, AB, AAAABD')
        self.assertEqual(self.sort_key(ZZZABE), 'A, AB, ZZZABE')
        self.assertEqual(self.sort_key(MMMACF), 'A, AC, MMMACF')

    def test_order_by_bases_multiple_inheritance(self):
        # Layerx      Layer1
        # |  \        /    \
        # |   \ Layer11    Layer12
        # |    \   | |      /    \
        # | Layer111 | Layer121  Layer122
        #  \        /
        #   \      /
        #   Layer112
        #
        class Layer1:
            pass

        class Layerx:
            pass

        class Layer11(Layer1):
            pass

        class Layer12(Layer1):
            pass

        class Layer111(Layerx, Layer11):
            pass

        class Layer121(Layer12):
            pass

        class Layer112(Layerx, Layer11):
            pass

        class Layer122(Layer12):
            pass
        self.assertEqual(
            self.order(Layer1, Layer11, Layer12, Layer111, Layer112, Layer121,
                       Layer122),
            'Layer1, Layer11, Layer111, Layer112, Layer12, Layer121, Layer122')
        self.assertEqual(self.order(Layer111, Layer12), 'Layer111, Layer12')
        self.assertEqual(self.order(Layerx, Layer1, Layer11, Layer112),
                         'Layer1, Layer11, Layerx, Layer112')
        self.assertEqual(self.sort_key(Layer111),
                         'Layer1, Layer11, Layerx, Layer111')
        self.assertEqual(self.sort_key(Layer12),
                         'Layer1, Layer12')

    def test_order_by_bases_reverse_tree(self):
        # E   D  F
        #  \ / \ /
        #   B   C
        #    \ /
        #     A
        class F:
            pass

        class E:
            pass

        class D:
            pass

        class C(D, F):
            pass

        class B(D, E):
            pass

        class A(B, C):
            pass
        self.assertEqual(self.order(A, B, C), 'B, C, A')
        self.assertEqual(self.order(A, B, C, D, E, F), 'D, E, B, F, C, A')

    def test_order_by_bases_mro_is_complicated(self):
        #  A  C  B  E  D
        # / \ | / \ | / \
        # \  K1     K2  /
        #  `---.    _--'
        #     \ \  / |
        #      \ K3 /
        #       \ |/
        #        ZZ
        class A:
            pass

        class B:
            pass

        class C:
            pass

        class D:
            pass

        class E:
            pass

        class K1(A, B, C):
            pass

        class K2(D, B, E):
            pass

        class K3(D, A):
            pass

        class ZZ(K1, K2, K3):
            pass
        self.assertEqual(self.order(K1, K2, K3, ZZ),
                         'K3, K2, K1, ZZ')
        # Sorting by reverse MRO, as computed by Python's MRO algorithm,
        # would put the layers in a different order: K3, K1, K2, ZZ.
        # Does that matter?  The class diagram is symmetric, so I think not.

    def test_FakeInputContinueGenerator_close(self):
        # multiprocessing (and likely other forkful frameworks) want to
        # close sys.stdin.  The test runner replaces sys.stdin with a
        # FakeInputContinueGenerator for some reason. It should be
        # closeable.

        f = runner.FakeInputContinueGenerator()
        f.close()


@unittest.skipIf(sys.warnoptions, "Only done if no user override")
class TestWarnings(unittest.TestCase):

    def test_warning_filter_default(self):
        # When we run tests, we run them with a 'default' simplefilter.
        # Note that this test will fail if PYTHONWARNINGS is set,
        # or a -W option was given, so we skip it
        import warnings

        # Save the current filters, ignoring the compiled regexes,
        # which can't be compared.
        old_filters = [(f[0], f[2], 4) for f in warnings.filters]
        with warnings.catch_warnings():
            # Set up just like the runner does
            warnings.simplefilter('default')
            warnings.filterwarnings('module',
                                    category=DeprecationWarning,
                                    message=r'Please use assert\w+ instead.')
            new_filters = [(f[0], f[2], 4) for f in warnings.filters]

        # For some reason, catch_warnings doesn't fully reset things,
        # and we wind up with some duplicate entries in new_filters
        self.assertEqual(set(old_filters), set(new_filters))

    def test_warnings_are_shown(self):
        import logging
        import warnings

        from zope.testing.loggingsupport import InstalledHandler

        handler = InstalledHandler("py.warnings", level=logging.WARNING)
        self.addCleanup(handler.uninstall)

        logging.captureWarnings(True)
        self.addCleanup(logging.captureWarnings, False)

        msg = "This should be visible by default"
        warnings.warn(msg, DeprecationWarning)

        self.assertEqual(1, len(handler.records))
        self.assertIn('DeprecationWarning', handler.records[0].getMessage())
        self.assertIn(msg, handler.records[0].getMessage())


class TestReprLines(unittest.TestCase):
    def test_unprintable(self):

        class C:
            def __repr__(self):
                raise Exception

        rl = runner.repr_lines(C(), max_width=10000)
        self.assertEqual(len(rl), 1)
        self.assertIn(".C'> instance at 0x", rl[0])

    def test_simple(self):
        rl = runner.repr_lines("")
        self.assertEqual(rl, [repr("")])

    def test_multiline(self):
        li = ["*" * 20 + " %d " % i + "*" * 20 for i in range(2)]
        rl = runner.repr_lines(li)
        self.assertEqual(rl, [
            "[" + repr(li[0]) + ",",
            " " + repr(li[1]) + "]"])

    def test_multiline_truncated(self):
        li = ["*" * 20 + "%d" % i + "*" * 20 for i in range(2)]
        rl = runner.repr_lines(li, max_lines=1)
        self.assertEqual(rl, [
            "[" + repr(li[0]) + ",",
            "..."])

    def test_line_truncated(self):
        rl = runner.repr_lines("*" * 20, max_width=10)
        self.assertEqual(rl, ["'******..."])



##############################################################################
#
# Copyright (c) 2003 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

import doctest
import unittest


x = 0
y = 0
z = 0


class TestA(unittest.TestCase):
    def setUp(self):
        global x
        x = 1

    def tearDown(self):
        global x
        x = 0

    def test_x1(self):
        self.assertEqual(x, 1)

    def test_y0(self):
        self.assertEqual(y, 0)

    def test_z0(self):
        self.assertEqual(z, 0)


class TestB(unittest.TestCase):
    def setUp(self):
        global y
        y = 1

    def tearDown(self):
        global y
        y = 0

    def test_y1(self):
        self.assertEqual(y, 1)

    def test_x0(self):
        self.assertEqual(x, 0)

    def test_z0(self):
        self.assertEqual(z, 0)


class TestNotMuch(unittest.TestCase):
    def test_1(self):
        pass

    def test_2(self):
        pass

    def test_3(self):
        pass


def setUp(test):
    test.globs['z'] = 1


def test_y0(self):
    """
    >>> y = 0
    >>> y
    0
    """


def test_x0(self):
    """
    >>> x = 0
    >>> x
    0
    """


def test_z1(self):
    """
    >>> z
    1
    """


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(TestA))
    suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(TestB))
    suite.addTest(
        unittest.defaultTestLoader.loadTestsFromTestCase(TestNotMuch))
    suite.addTest(doctest.DocTestSuite(setUp=setUp))
    suite.addTest(doctest.DocFileSuite('../sampletests.rst',
                                       setUp=setUp))
    return suite



##############################################################################
#
# Copyright (c) 2004-2008 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Debug functions

"""

import doctest
import io
import pdb  # noqa: T100 import for pdb found
import sys
import threading
import traceback


try:
    import ipdb  # noqa: T100 import for ipdb found
    _ipdb_state = threading.local()
except ImportError:
    ipdb = None

import zope.testrunner.interfaces


def _use_ipdb():
    """Check whether ipdb is usable."""
    global _ipdb_ok
    if ipdb is None:
        return False
    if getattr(_ipdb_state, 'ok', None) is None:
        _ipdb_state.ok = False
        if hasattr(sys.stdin, 'isatty') and sys.stdin.isatty():
            try:
                sys.stdin.fileno()
            except (AttributeError, io.UnsupportedOperation):
                pass
            else:
                _ipdb_state.ok = True
    return _ipdb_state.ok


def post_mortem(exc_info):
    err = exc_info[1]
    if isinstance(err, (doctest.UnexpectedException, doctest.DocTestFailure)):

        if isinstance(err, doctest.UnexpectedException):
            exc_info = err.exc_info

            # Print out location info if the error was in a doctest
            if exc_info[2].tb_frame.f_code.co_filename == '<string>':
                print_doctest_location(err)

        else:
            print_doctest_location(err)
            # Hm, we have a DocTestFailure exception.  We need to
            # generate our own traceback
            try:
                exec(('raise ValueError'
                      '("Expected and actual output are different")'
                      ), err.test.globs)
            except BaseException:
                exc_info = sys.exc_info()

    print(''.join(traceback.format_exception_only(exc_info[0], exc_info[1])))
    if _use_ipdb():
        ipdb.post_mortem(exc_info[2])
    else:
        pdb.post_mortem(exc_info[2])
    raise zope.testrunner.interfaces.EndRun()


def print_doctest_location(err):
    # This mimics pdb's output, which gives way cool results in emacs :)
    filename = err.test.filename
    if filename.endswith('.pyc'):
        filename = filename[:-1]
    print(
        "> {}({})_()".format(
            filename,
            err.test.lineno +
            err.example.lineno +
            1))



##############################################################################
#
# Copyright (c) 2004-2008 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Filter which tests to run.
"""

import re

import zope.testrunner.feature


UNITTEST_LAYER = 'zope.testrunner.layer.UnitTests'


class Filter(zope.testrunner.feature.Feature):
    """Filters and orders all tests registered until now."""

    active = True

    def global_setup(self):
        layers = self.runner.tests_by_layer_name
        options = self.runner.options

        if UNITTEST_LAYER in layers:
            # We start out assuming unit tests should run and look for reasons
            # why they shouldn't be run.
            should_run = True
            if (not options.non_unit):
                if options.layer:
                    accept = build_filtering_func(options.layer)
                    should_run = accept(UNITTEST_LAYER)
                else:
                    should_run = True
            else:
                should_run = False

            if not should_run:
                layers.pop(UNITTEST_LAYER)

        if self.runner.options.resume_layer is not None:
            for name in list(layers):
                if name != self.runner.options.resume_layer:
                    layers.pop(name)
            if not layers:
                self.runner.options.output.error_with_banner(
                    "Cannot find layer %s" % self.runner.options.resume_layer)
                self.runner.errors.append(
                    ("subprocess failed for %s" %
                        self.runner.options.resume_layer,
                     None))
        elif self.runner.options.layer:
            accept = build_filtering_func(self.runner.options.layer)
            for name in list(layers):
                if not accept(name):
                    # No pattern matched this name so we remove it
                    layers.pop(name)

        if (self.runner.options.verbose and
                not self.runner.options.resume_layer):
            if self.runner.options.all:
                msg = "Running tests at all levels"
            else:
                if self.runner.options.only_level is None:
                    msg = (
                        "Running tests at level %d" %
                        self.runner.options.at_level)
                else:
                    msg = (
                        "Running tests only at level %d" %
                        self.runner.options.only_level)
            self.runner.options.output.info(msg)

    def report(self):
        if not self.runner.do_run_tests:
            return
        if self.runner.options.resume_layer:
            return
        if self.runner.options.verbose:
            self.runner.options.output.tests_with_errors(self.runner.errors)
            self.runner.options.output.tests_with_failures(
                self.runner.failures)


def build_filtering_func(patterns):
    """Build a filtering function from a set of patterns

    Patterns are understood as regular expressions, with the additional feature
    that, prefixed by "!", they create a "don't match" rule.

    This returns a function which returns True if a string matches the set of
    patterns, or False if it doesn't match.

    """

    selected = []
    unselected = []

    for pattern in patterns:
        if pattern.startswith('!'):
            store = unselected.append
            pattern = pattern[1:]
        else:
            store = selected.append

        store(re.compile(pattern).search)

    if not selected and unselected:
        # If there's no selection patterns but some un-selection patterns,
        # suppose we want everything (that is, everything that matches '.'),
        # minus the un-selection ones.
        selected.append(re.compile('.').search)

    def accept(value):
        return (any(search(value) for search in selected) and not
                any(search(value) for search in unselected))

    return accept



##############################################################################
#
# Copyright (c) 2004 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
import doctest
import unittest


class TestSomething(unittest.TestCase):

    def test_set_trace1(self):
        x = 1
        import pdb  # noqa: T100 import for pdb found
        pdb.set_trace()  # noqa: T100 pdb.set_trace found
        y = x  # noqa: F841

    def test_set_trace2(self):
        f()

    def test_post_mortem1(self):
        x = 1  # noqa: F841
        raise ValueError

    def test_post_mortem2(self):
        g()

    def test_post_mortem_failure1(self):
        x = 1
        y = 2
        assert x == y

    @unittest.skip("skipped test")
    def test_skipped(self):
        self.fail('test should have been skipped')


def f():
    x = 1
    import pdb  # noqa: T100 import for pdb found
    pdb.set_trace()  # noqa: T100 pdb.set_trace found
    y = x  # noqa: F841


def g():
    x = 1  # noqa: F841
    raise ValueError


def set_trace3(self):
    """
    >>> x = 1
    >>> if 1:
    ...     import pdb; pdb.set_trace()
    ...     y = x
    """


def set_trace4(self):
    """
    >>> f()
    """


def post_mortem3(self):
    """
    >>> x = 1
    >>> raise ValueError
    """


def post_mortem4(self):
    """
    >>> g()
    """


def post_mortem_failure2():
    """
    >>> x = 1
    >>> x
    2
    """


def test_suite():
    return unittest.TestSuite((
        doctest.DocTestSuite(),
        unittest.defaultTestLoader.loadTestsFromTestCase(TestSomething),
        doctest.DocFileSuite('set_trace5.rst'),
        doctest.DocFileSuite('set_trace6.rst'),
        doctest.DocFileSuite('post_mortem5.rst'),
        doctest.DocFileSuite('post_mortem6.rst'),
        doctest.DocFileSuite('post_mortem_failure.rst'),
    ))


if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')
